# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
from report import report_sxw
from osv import osv
import pooler
import mx.DateTime

class shipping(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(shipping, self).__init__(cr, uid, name, context=context)
        self.localcontext.update({
            'time': time,
            'date_only': self.date_only,
            'format_quantity_digits': self.format_quantity_digits,
            'invoice_numbers': self.invoice_numbers,
            'combined_lines': self.combined_lines
        })

    def format_quantity_digits(self,line):
        import re
        rounding_str = str(line.product_uom.rounding)
        round_decimals = re.search('\.(.*[1-9])',rounding_str)
        decimals = 0
        if ( round_decimals ):
          decimals = len(round_decimals.group(1))
        return decimals

    def invoice_numbers(self,picking):
        '''
        Return a comma separated list of invoice numbers
        associated with this stock.picking entry
        '''
        invoice_numbers = []

        # Important: Until the user has done the following, the
        # invoice number will not be available:
        #
        # 1. Clicked on "Create Invoice"
        # 2. "Created", not left the invoice in draft mode
        # 
        if picking.sale_id:
            for invoice in picking.sale_id.invoice_ids:
                if invoice.number:
                    invoice_numbers.append( invoice.number )
        if len(invoice_numbers) > 0:
            return ", ".join(invoice_numbers)
        return "N/A"

    def date_only(self,date):
        '''
        Returns just the date portion of a date-time string
        '''
        # May need to call formatLang to populate the lang_dict.
        if 'date_format' not in self.lang_dict or 'time_format' not in self.lang_dict:
            formatted = self.formatLang(date, date=True)
        if 'date_format' not in self.lang_dict or 'time_format' not in self.lang_dict:
            return formatted
        parse_format = self.lang_dict['date_format'] + " " + self.lang_dict['time_format']
        date = mx.DateTime.strptime(date,parse_format)
        return date.strftime(self.lang_dict['date_format'])

    
    def combined_lines(self, picking):
        picking_obj = pooler.get_pool(self.cr.dbname).get('stock.picking')
        return picking_obj._combined_lines(picking)

report_sxw.report_sxw('report.sale.shipping.combined.lines',
                      'stock.picking',
                      'addons/combined_lines_delivery/report/shipping.rml',
                      parser=shipping)
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
