import re

from osv import fields, osv
from tools.translate import _

class stock_picking(osv.osv):
    _name = 'stock.picking'
    _inherit = 'stock.picking'
    
    _columns = {
        'checklist_printed': fields.boolean(
                'Check-list printed',
                help = 'Has an order check-list been printed?')
    }
    
    def _prepare_invoice_line(self, 
                              cr, 
                              uid, 
                              group, 
                              picking, 
                              move_line, 
                              invoice_id,
                              invoice_vals, 
                              context=None):
        """ Builds the dict containing the values for the invoice line. This
            version just copies the grouping field from the stock move.
            @param group: True or False
            @param picking: picking object
            @param: move_line: move_line object
            @param: invoice_id: ID of the related invoice
            @param: invoice_vals: dict used to created the invoice
            @return: dict that will be used to create the invoice line
        """
        
        vals = super(stock_picking, self)._prepare_invoice_line(cr,
                                                                uid,
                                                                group,
                                                                picking,
                                                                move_line,
                                                                invoice_id,
                                                                invoice_vals,
                                                                context)
        vals['grouping'] = move_line.grouping
        return vals
    

    def _prepare_line_to_combine(self, line):
        """ Prepare stock move line before it gets combined. """
        line.prodlot_name = line.prodlot_id and line.prodlot_id.name or ''
        line.variants = line.product_id and line.product_id.variants or ''
        line.revision = line.product_id and line.product_id.revision or ''
        line.sale_notes = line.sale_line_id and line.sale_line_id.notes or ''

    def _prepare_product_builder(self, builder):
        """ Configure product builder with fields to match, copy, etc. """
        builder.add_fields_to_match(['product_qty', 'product_uom'])
        builder.add_fields_to_max(['promise_date'])
        builder.add_fields_to_hyphenate(['prodlot_name', 
                                         'variants', 
                                         'revision'])
        builder.add_fields_to_concatenate(['sale_notes', 'note'])
        builder.add_fields_to_sum(['weight'])

    def _combined_lines(self, picking):
        product_obj = self.pool.get('product.product')
        
        # Calculate some fields before grouping
        builder = product_obj._create_builder(picking.move_lines)
        self._prepare_product_builder(builder)
        for line in builder.input_lines:
            self._prepare_line_to_combine(line)
        
        # Now group
        combined_lines = builder.combine()
        if builder.warnings:
            raise osv.except_osv(_('Invalid grouping!'), builder.warnings)
        
        name_pattern = (
            r'^\[([^\[\]]*)\]\s*' # [code] plus optional whitespace
            r'([^\[\]]*)$') # anything else, except square brackets
        for line in combined_lines:
            match = re.match(name_pattern, line.name)
            if match:
                line.code = match.group(1)
            else:
                line.code = ''
        
        return combined_lines

stock_picking()

class stock_move(osv.osv):
    _name = 'stock.move'
    _inherit = 'stock.move'
    _order = 'grouping, id'
    
    _columns = {
        'grouping': fields.char(
            'Grouping',
            size=20,
            help='Used to sort line items alphabetically and group them. '
                'Before the period is the group name, the suffix orders within '
                'a group.'),
    }

stock_move()
