from osv import fields,osv
import time
from datetime import datetime, date
from tools.translate import _

class crm_helpdesk(osv.osv):
    _inherit = 'crm.helpdesk'
    _columns = {
        'last_reply': fields.datetime('Last Reply'),        
    }
    _defaults = {    
        'last_reply': lambda *a: time.strftime('%Y-%m-%d %H:%M:%S'),    
    }
    _order = 'last_reply desc'
    
    def init(self, cr):
        cr.execute("""
            update crm_helpdesk 
            set last_reply=date_trunc('second',coalesce(write_date,create_date)) 
            where last_reply <> create_date
        """)
        cr.execute("""
            update crm_helpdesk 
            set last_reply = date_trunc('second',last) 
            from (
                select max(create_date) as last, res_id 
                from mail_message 
                where model = 'crm.helpdesk' 
                group by res_id
              ) as subq 
            where 
              id = subq.res_id
              and last_reply <> create_date
            """)

class crm_add_history(osv.osv_memory):
    """Adds a new history entry to the case."""
    _inherit = 'crm.add.history'
    
    def action_add(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        model = 'crm.helpdesk'
        case_pool = self.pool.get(model)
        mail_message = self.pool.get('mail.message')

        for obj in self.browse(cr, uid, ids, context=context):
            case_list = case_pool.browse(cr, 
                                         uid, 
                                         context['active_ids'],
                                         context=context)
            case = case_list[0]
            data = {
                'subject': 'Historize',
                'user_id': uid,
                'model' : model,
                'res_id': case.id,
                'date': time.strftime('%Y-%m-%d %H:%M:%S'),
                'body_text': obj.body,
                'partner_id': case.partner_id.id,
                'entry_type_id': obj.entry_type_id.id,
                'state' : 'received',
            }
            mail_message.create(cr, uid, data, context=context)
            if obj.state == 'unchanged':
                pass
            elif obj.state == 'done':
                case_pool.case_close(cr, uid, [case.id])
            elif obj.state == 'draft':
                case_pool.case_reset(cr, uid, [case.id])
            elif obj.state in ['cancel', 'open', 'pending']:
                act = 'case_' + obj.state
                getattr(case_pool, act)(cr, uid, [case.id])
            #raise osv.except_osv(_('Warning !'),_("Inside !"))
            query = _("update crm_helpdesk set last_reply = '%s' where id=%s")%(time.strftime('%Y-%m-%d %H:%M:%S'),case.id)
            cr.execute(query)

        return {'type': 'ir.actions.act_window_close'}
    
