import time
import datetime
from osv import fields, osv
from tools.misc import DEFAULT_SERVER_DATETIME_FORMAT
from tools.translate import _

class stock_partial_picking(osv.osv_memory):
    _inherit = "stock.partial.picking"

    _columns = {
        'date_cleared': fields.date('Date Cleared to Ship'),
        'must_clear': fields.boolean('Must Clear')
     }

    def default_get(self, cr, uid, fields, context=None):
        if context is None: 
            context = {}
        res = super(stock_partial_picking, self).default_get(cr, 
                                                             uid, 
                                                             fields, 
                                                             context)
        
        picking_ids = context.get('active_ids', [])
        if not res or len(picking_ids) != 1:
            return res
        if 'date_cleared' in fields or 'must_clear' in fields:
            picking = self.pool.get('stock.picking').browse(cr, 
                                                            uid, 
                                                            picking_ids[0], 
                                                            context)
        if 'date_cleared' in fields:
            res.update(date_cleared=picking.date_cleared)
        if 'must_clear' in fields:
            res.update(must_clear=(picking.type == 'out'))
        return res

    def do_partial(self, cr, uid, ids, context=None):
        partial = self.browse(cr, uid, ids[0], context)
        
#        print datetime.datetime.today().strftime('%Y-%m-%d %H:%M:%S %z')
        now = datetime.datetime.today().strftime('%Y-%m-%d %H:%M:%S')
        if partial.date_cleared > now:
            raise osv.except_osv(_('Warning!'), _('Not cleared to ship yet.'))
        
        picking_obj = self.pool.get('stock.picking')
        picking_obj.write(cr, 
                          uid, 
                          partial.picking_id.id, 
                          {'date_cleared': partial.date_cleared})

        return super(stock_partial_picking, self).do_partial(cr, 
                                                             uid, 
                                                             ids, 
                                                             context)
        
