# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution    
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time
import wizard
import pooler
from osv.osv import except_osv
from osv import fields,osv
import netsvc
from tools.translate import _

inventory_form = """<?xml version="1.0"?>
<form string="Fill Inventory">
    <separator colspan="4" string="Fill from a product category" />
    <field name="category_id"/>
    <newline/>
    <field name="include_components"/>
    <newline/>
    <field name="include_child_categories"/>
    <newline/>

</form>
"""

inventory_fields = {
    'category_id' : {
        'string':'Product Category',
        'type':'many2one',
        'relation':'product.category',
        'required':True,
        'help': 'All products from this category will be placed in the ' +
            'count, along with their current stock levels.'
    },
    'include_components' : {
        'string': 'Include Components', 
        'type': 'boolean',
        'help': 'Include all components and subcomponents in the count.',
    },
    'include_child_categories' : {
        'string': 'Include Child Categories', 
        'type': 'boolean',
        'help': 'Include all child categories and other descendants.',
    },
}




def _get_component_ids(cr, uid, bom_obj, product_id):
    bom_id = bom_obj._bom_find(cr, 
        uid, 
        product_id, 
        product_uom=None)
    if bom_id:
        bom = bom_obj.browse(cr, uid, bom_id)
        exploded = bom_obj._bom_explode(
            cr, 
            uid, 
            bom, 
            factor=1, 
            properties=[])
        component_lines = exploded[0]
        return [line['product_id'] for line in component_lines]
    return []

def _write_inventory(
    cr, 
    uid, 
    inventory_line_obj, 
    inventory_id,
    inventory, 
    product_id,
    uom,
    default_location_id):
    filtered = [
        (location_id, qty) 
        for location_id, qty in inventory.iteritems() 
        if qty != 0]
    if not filtered:
        filtered = [(default_location_id, 0)]
    for location_id, qty in filtered:
        line_ids = inventory_line_obj.search(
            cr,
            uid,
            [
                ('inventory_id', '=', inventory_id),
                ('location_id', '=', location_id),
                ('product_id', '=', product_id),
                ('product_uom','=',uom.id),
                ('product_qty','=',qty)])
        if not len(line_ids):
            new_row = {
                'inventory_id': inventory_id,
                'location_id': location_id,
                'product_id': product_id,
                'product_uom': uom.id,
                'product_qty': qty
            }
            inventory_line_obj.create(cr, uid, new_row)

def _fill_inventory(self, cr, uid, data, context):
    pool = pooler.get_pool(cr.dbname)
    inventory_line_obj = pool.get('stock.inventory.line')
    product_obj = pool.get('product.product')
    bom_obj = pool.get('mrp.bom')
    warehouse_obj = pool.get('stock.warehouse')
    location_obj = pool.get('stock.location')
    uom_obj = pool.get('product.uom')
    
    category_id = data['form']['category_id']
    if data['form']['include_child_categories']:
        criteria = [('product_tmpl_id.categ_id.parent_id', 
                     'child_of', 
                     [category_id])]
    else:
        criteria = [('product_tmpl_id.categ_id', 
                     '=', 
                     category_id)]
    product_ids_to_visit = product_obj.search(cr, uid, criteria)
    product_types = {}
    product2uom = {} # product_id -> uom browse object
    uoms = {} # uom_id -> uom_browse object
    while product_ids_to_visit:
        product_id = product_ids_to_visit.pop()
        if product_id not in product_types:
            product = product_obj.browse(cr, uid, product_id)
            if product.product_tmpl_id.categ_id.skip_counts:
                product_types[product_id] = 'skipped'
            else:
                product_types[product_id] = product.product_tmpl_id.type
            product2uom[product_id] = product.uom_id
            uoms[product.uom_id.id] = product.uom_id
            if (data['form']['include_components'] 
                and product.product_tmpl_id.supply_method == 'produce'):
                component_ids = _get_component_ids(cr, uid, bom_obj, product_id)
                product_ids_to_visit.extend(component_ids)
    product_ids = set([id for id, type in product_types.iteritems() if type == 'product'])     

    if not product_ids:
        raise wizard.except_wizard(_('Message !'), _('No products in the category.'))

    location_ids = []
    warehouse_ids = warehouse_obj.search(cr, uid, [], context=context)
    for warehouse in warehouse_obj.browse(cr, uid, warehouse_ids, context=context):
        location_ids.append(warehouse.lot_stock_id.id)
    default_location_id = location_ids[0]
    child_location_ids = location_obj.search(cr, uid, [('location_id', 'child_of', location_ids)])
    location_ids = child_location_ids or location_ids

    sql = """
    select
            product_id, 
            location_id, 
            location_dest_id, 
            sum(product_qty) as qty, 
            product_uom
    from stock_move
    where product_id in %s
    and location_id != location_dest_id
    and (location_id in %s or location_dest_id in %s)
    and state = 'done'
    group by product_id, product_uom, location_id, location_dest_id
    order by product_id, product_uom, location_id, location_dest_id
    """
    cr.execute(
        sql,
        (tuple(product_ids), tuple(location_ids), tuple(location_ids)))
    results = cr.fetchall()

    current_product_id = None
    for product_id, location_id, location_dest_id, qty, uom_id in results:
        if product_id != current_product_id:
            if current_product_id != None:
                _write_inventory(
                    cr, 
                    uid, 
                    inventory_line_obj, 
                    data['id'],
                    inventory, 
                    current_product_id,
                    product2uom[current_product_id],
                    default_location_id)
                product_ids.remove(current_product_id)
                
            current_product_id = product_id
            inventory = {} #location_id -> qty
        product_uom = product2uom[product_id]
        if uom_id != product_uom.id:
            uom = uoms.get(uom_id)
            if not uom:
                uom = uom_obj.browse(cr, uid, uom_id)
                uoms[uom_id] = uom
            qty = uom_obj._compute_qty_obj(
                cr, 
                uid, 
                uom, 
                qty,
                product_uom)
        if location_id in location_ids:
            inventory[location_id] = inventory.get(location_id, 0) - qty
        if location_dest_id in location_ids:
            inventory[location_dest_id] = inventory.get(location_dest_id, 0) + qty
    _write_inventory(
        cr, 
        uid, 
        inventory_line_obj, 
        data['id'],
        inventory, 
        current_product_id,
        product2uom[current_product_id],
        default_location_id)
    product_ids.remove(current_product_id)
    
    inventory = {}
    for product_id in product_ids:
        _write_inventory(
            cr, 
            uid, 
            inventory_line_obj, 
            data['id'],
            inventory, 
            product_id,
            product2uom[product_id],
            default_location_id)

    return {}

class fill_inventory(wizard.interface):
    states = {
        'init' : {
            'actions' : [],
            'result' : {'type' : 'form',
                    'arch' : inventory_form,
                    'fields' : inventory_fields,
                    'state' : [('end', 'Cancel'),('fill_inventory', 'Fill Inventory') ]}
        },
        'fill_inventory' : {
            'actions' : [],
            'result' : {'type' : 'action', 'action': _fill_inventory, 'state' : 'end'}
        },
    }
fill_inventory("inventory_by_category.fill_inventory")



# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
