from osv import osv

class mrp_bom(osv.osv):
    _inherit = 'mrp.bom'
    
    def _bom_explode(self, 
                     cr, 
                     uid, 
                     bom, 
                     factor, 
                     properties=None, 
                     addthis=False, 
                     level=0, 
                     routing_id=False):
        """ Finds Products and Work Centers for related BoM for manufacturing 
        order. This override checks for a special case. If a BOM line's product
        has a phantom BOM, then behave as if the line itself were a phantom,
        and explode the component product's BOM.
        @param bom: BoM of particular product.
        @param factor: Factor of product UoM.
        @param properties: A List of properties Ids.
        @param addthis: If BoM found then True else False.
        @param level: Depth level to find BoM lines starts from 10.
        @return: result: List of dictionaries containing product details.
                 result2: List of dictionaries containing Work Center details.
        """
        if properties is None:
            properties = []
            
        if not bom.bom_lines:
            newbom_id = self._bom_find(cr, 
                                       uid, 
                                       bom.product_id.id, 
                                       bom.product_uom.id, 
                                       properties)
            
            if newbom_id:
                newbom = self.browse(cr, uid, newbom_id)
                if newbom.type == 'phantom':
                    return self._bom_explode(cr, 
                                             uid, 
                                             newbom, 
                                             factor*bom.product_qty, 
                                             properties, 
                                             addthis=True, 
                                             level=level+10)
        return super(mrp_bom, self)._bom_explode(cr,
                                                 uid,
                                                 bom,
                                                 factor,
                                                 properties,
                                                 addthis,
                                                 level,
                                                 routing_id)
mrp_bom()
