class Snapshot(object):
    """ Details used for calculating build capacities.
    """
    NEW, CALCULATING, CALCULATED = range(3)

    def __init__(self, product_id):
        self.product_id = product_id
        self.state = Snapshot.NEW
        self.virtual_available = 0.0
        self.procurement_qty = 0.0
        self.purchase_qty = 0.0
        self.production_qty = 0.0
        self.build_capacity = 0.0
        self.sales_capacity = 0.0
        self.committed = 0.0
        self.bom_id = None
        self.is_slow_to_produce = False
        self.can_be_sold = False
        self.daily_sales = 0.0
        self.uom = None
        self.product_type = None
        self.delay = 0.0
        self.safety_days = 0.0
        self.supply_method = None
        self.replacement = None
        
        # List of components used to build this product:
        # [(component_id, qty_consumed, consumed_uom)]
        # Nested components are not exploded.
        self.components = []
        self.qty_produced = 1
    
    def has_changed(self, product):
        """ Have any of the stored fields changed? """
        delta = 0.001
        return (
            (abs(self.procurement_qty - product.procurement_qty) > delta) or
            (abs(self.purchase_qty - product.purchase_qty) > delta) or
            (abs(self.production_qty - product.production_qty) > delta) or
            (abs(self.build_capacity - product.build_capacity) > delta) or
            (abs(self.sales_capacity - product.sales_capacity) > delta) or
            (abs(self.committed - product.committed) > delta))
    
    def get_data(self):
        """ Retrieve all the data fields in a dictionary for updating the table.
        """
        
        return {
            'procurement_qty': self.procurement_qty,
            'purchase_qty': self.purchase_qty,
            'production_qty': self.production_qty,
            'build_capacity': self.build_capacity,
            'sales_capacity': self.sales_capacity,
            'committed': self.committed
        }
