import re

from osv import osv
from tools import float_compare
from tools.translate import _

class sale_order_line(osv.osv):
    _inherit = 'sale.order.line'

    def product_id_change(self, 
                          cr, 
                          uid, 
                          ids, 
                          pricelist, 
                          product, 
                          qty=0,
                          uom=False, 
                          qty_uos=0, 
                          uos=False, 
                          name='', 
                          partner_id=False,
                          lang=False, 
                          update_tax=True, 
                          date_order=False, 
                          packaging=False, 
                          fiscal_position=False, 
                          flag=False, 
                          context=None):
        result = super(sale_order_line, self).product_id_change(cr,
                                                                uid, 
                                                                ids, 
                                                                pricelist, 
                                                                product, 
                                                                qty,
                                                                uom, 
                                                                qty_uos, 
                                                                uos, 
                                                                name, 
                                                                partner_id,
                                                                lang, 
                                                                update_tax, 
                                                                date_order, 
                                                                packaging, 
                                                                fiscal_position, 
                                                                flag, 
                                                                context)
        
        # Remove warning about low stock, because we produce many products
        # after receiving orders.
        warning_key = 'warning'
        message_key = 'message'
        warning = result.setdefault(warning_key, {})
        old_message = warning.get(message_key, '')
        pattern = r'\s*Not enough stock !.*\(without reservations\)\s*'
        new_message = re.sub(pattern, '', old_message, flags=re.DOTALL)

        # Now check build capacity instead.
        if product:
            product_uom_obj = self.pool.get('product.uom')
            product_obj = self.pool.get('product.product')
    
            product_rec = product_obj.browse(cr, uid, product, context=context)
    
            uom2 = False
            if uom:
                uom2 = product_uom_obj.browse(cr, uid, uom)
                if product_rec.uom_id.category_id.id != uom2.category_id.id:
                    uom = False
    
            if not uom2:
                uom2 = product_rec.uom_id
            compare_qty = float_compare(
                product_rec.sales_capacity, 
                qty / uom2.factor * product_rec.uom_id.factor, 
                precision_rounding=product_rec.uom_id.rounding)
            if (product_rec.type == 'product' and 
                int(compare_qty) == -1 and 
                product_rec.procure_method == 'make_to_stock'):
                
                warn_msg = (
                    _("You plan to sell %.2f %s but the sales capacity is " +
                      "only %.2f %s. This may mean that you can't fulfil " +
                      "the order.") % (qty, 
                                       uom2.name,
                                       product_rec.sales_capacity,
                                       product_rec.uom_id.name))
                if new_message:
                    new_message += '\n\n'
                new_message += _("Not enough stock: ") + warn_msg + "\n\n"

        if new_message:
            warning[message_key] = new_message
        else:
            del result[warning_key]
        return result

sale_order_line()
