class stockout_snapshot:
    """ This is used within the stockout_scanner to represent a record in the
    product_stockout table.
    
    >>> component_id = 2
    >>> stockout_snapshot(component_id)
    stockout_snapshot(2)
    """
    def __init__(self, component_id):
        self.component_id = component_id
        self.consumption = 1.0
        self.component_procurement_days = 0.0
        self.component_to_product_days = 0.0
    
    def __repr__(self):
        return "stockout_snapshot(%d)" % self.component_id
    
    def has_changed(self, stockout):
        """ Have any of the stored fields changed? """
        delta = 0.001
        return (
            (abs(self.consumption - stockout.consumption) > delta) or
            (abs(self.component_procurement_days - stockout.component_procurement_days) > delta) or
            (abs(self.component_to_product_days - stockout.component_to_product_days) > delta))
    
    def get_data(self):
        """ Retrieve all the data fields in a dictionary for updating the table.
        """
        
        return {
            'consumption': self.consumption,
            'component_procurement_days': self.component_procurement_days,
            'component_to_product_days': self.component_to_product_days,
        }

if __name__ == "__main__":
    import doctest
    
    failures, tests = doctest.testmod()
    if failures == 0:
        print "Success."
