from osv import fields, osv

class availability_check(osv.osv_memory):
    _name = "availability.check"
    _description = "Availability Check"

    def _generate_report(self, cr, uid, context=None):
        if context['active_model'] == 'mrp.production':
            return self._generate_production_report(cr, uid, context)
        else:
            return self._generate_picking_report(cr, uid, context)

    def _generate_production_report(self, cr, uid, context):
        stock_picking_obj = self.pool.get('stock.picking')
        production_obj = self.pool.get('mrp.production')
    
        production_ids = context['active_ids']
        picking_ids = stock_picking_obj.search(
            cr,
            uid,
            [('production_ids', 'in', production_ids),
             ('state', '=', 'confirmed')])
        stock_picking_obj._check_procurements(cr, uid, picking_ids, context)
        report = ''
        for production in production_obj.browse(cr, uid, production_ids, context):
            if report:
                report += '\n\n'
            if production.state == 'confirmed':
                stock_picking = production.picking_id
                if stock_picking.state == 'confirmed':
                    confirmed = []
                    for move in stock_picking.move_lines:
                        if move.state in ('confirmed', 'waiting'):
                            product = move.product_id
                            description = '[%s] %s' % (product.default_code, product.name)
                            confirmed.append(description)
                    report += production.name + ' still waiting for:\n'
                    report += '\n'.join(confirmed)
                elif stock_picking.state == 'assigned' and not stock_picking.auto_picking:
                    report += production.name + ' is ready to be manually packed'
                else:
                    pack_state_names = dict(stock_picking_obj._columns['state'].selection)
                    pack_state_name = pack_state_names[stock_picking.state]
                    report += "%s is waiting for a stock picking that is %s" % (
                        production.name, 
                        pack_state_name)
            else:
                state_names = dict(production_obj._columns['state'].selection)
                state_name = state_names[production.state]
                report += "%s %s" % (production.name, state_name)
        return report
    
    def _generate_picking_report(self, cr, uid, context):
        stock_picking_obj = self.pool.get('stock.picking')
    
        move_ids = context['active_ids']
        picking_ids = stock_picking_obj.search(
            cr,
            uid,
            [('move_lines', 'in', move_ids),
             ('state', '=', 'confirmed')])
        stock_picking_obj.action_assign(cr, uid, picking_ids)
        report = ''
        for stock_picking in stock_picking_obj.browse(cr, uid, picking_ids, context):
            if report:
                report += '\n\n'
            if stock_picking.state == 'confirmed':
                confirmed = []
                for move in stock_picking.move_lines:
                    if move.state == 'confirmed':
                        product = move.product_id
                        description = '[%s] %s' % (product.default_code, product.name)
                        confirmed.append(description)
                report += stock_picking.name + ' still waiting for:\n'
                report += '\n'.join(confirmed)
            else:
                pack_state_names = dict(stock_picking_obj._columns['state'].selection)
                pack_state_name = pack_state_names[stock_picking.state]
                report += "%s is %s" % (stock_picking.name, pack_state_name)
        return report
    
    _columns = {'report': fields.text('Report', readonly=True)}
    _defaults = {'report': _generate_report}
    
availability_check()
