from osv import fields,osv

class mrp_production(osv.osv):
    _inherit = 'mrp.production'
    
    def action_produce_all(self, cr, uid, ids, context=None):
        orders = self.pool.get('mrp.production').browse(cr, 
                                                        uid,
                                                        ids, 
                                                        context=context)
        for prodn_order in orders:
            qty_done = 0.0
            for move in prodn_order.move_created_ids2:
                if move.product_id == prodn_order.product_id:
                    if not move.scrapped:
                        qty_done += move.product_qty
            qty_remaining = ((prodn_order.product_qty - qty_done) or 
                             prodn_order.product_qty)
            self.action_produce(cr,
                                uid, 
                                prodn_order.id, 
                                qty_remaining, 
                                'consume_produce', 
                                context=context)
        return True
            
    def action_compute(self, cr, uid, ids, properties=[], context=None):
        """ Computes bills of material of a product, but improves on the
        official version by checking that there are still enough raw materials
        for this production order with the new quantity.
        Before calling this, update mrp_production.product_qty.
        This method will:
        - Delete *production order* lines.
        - Cancel raw material and finished product moves (excluding moves
            on the raw material packing list).
        - Undo the raw material packing list by cancelling it or reversing it.
        - Recalculate the *production order* lines.
        After calling this, send "button_recreate" signal to the workflow to 
            generate a new packing list and other moves.
        @param properties: List containing dictionaries of properties.
        @return: No. of products.
        """
        results = []
        bom_obj = self.pool.get('mrp.bom')
        uom_obj = self.pool.get('product.uom')
        prod_line_obj = self.pool.get('mrp.production.product.line')
        workcenter_line_obj = self.pool.get('mrp.production.workcenter.line')
        picking_obj = self.pool.get('stock.picking')
        move_obj = self.pool.get('stock.move')
        for production in self.browse(cr, uid, ids):
            cr.execute('delete from mrp_production_product_line where production_id=%s', (production.id,))
            cr.execute('delete from mrp_production_workcenter_line where production_id=%s', (production.id,))
            
            move_ids = (
                [move.id for move in production.move_lines] +
                [move.id for move in production.move_created_ids])
            
            # Have to switch to draft before deleting to avoid error.
            # If we don't delete the moves, they show up as consumed products.
            move_obj.write(cr, uid, move_ids, {'state': 'draft'})
            move_obj.unlink(cr, uid, move_ids)
            
            picking_obj.undo(cr, uid, production.picking_id.id, 'assigned')
            
            bom_point = production.bom_id
            bom_id = production.bom_id.id
            if not bom_point:
                bom_id = bom_obj._bom_find(cr, uid, production.product_id.id, production.product_uom.id, properties)
                if bom_id:
                    bom_point = bom_obj.browse(cr, uid, bom_id)
                    updates = self._prepare_production_from_bom(cr, 
                                                                uid, 
                                                                bom_point, 
                                                                context)
                    self.write(cr, 
                               uid, 
                               [production.id], 
                               updates)

            if not bom_id:
                raise osv.except_osv(_('Error'), _("Couldn't find a bill of material for this product."))
            factor = uom_obj._compute_qty(cr, uid, production.product_uom.id, production.product_qty, bom_point.product_uom.id)
            res = bom_obj._bom_explode(cr, uid, bom_point, factor / bom_point.product_qty, properties, routing_id=production.routing_id.id)
            results = res[0]
            results2 = res[1]
            for line in results:
                line['production_id'] = production.id
                prod_line_obj.create(cr, uid, line)
            for line in results2:
                line['production_id'] = production.id
                workcenter_line_obj.create(cr, uid, line)
        return len(results)
    
    def _prepare_production_from_bom(self, cr, uid, bom, context=None):
        """ Prepare production order fields to update based on a BOM.
        
        bom - a browse object to copy details from
        returns a dictionary of field names and their new values.
        """
        routing_id = bom.routing_id.id or False
        return {'bom_id': bom.id, 
                'routing_id': routing_id}

mrp_production()
