from osv import osv
import netsvc

class stock_picking(osv.osv):
    _inherit = 'stock.picking'

    def undo(self, cr, uid, id, reversal_state='confirmed'):
        """ Cancel or reverse a stock picking.
        
        If the picking is in progress, cancel it. If it's done, generate a new
        picking that reverses all the stock moves. reversal_state tells it
        what state to create the reversal moves in. If you want to force the
        moves, use 'assigned'.
        """
        if not id:
            return
        
        picking = self.browse(cr, uid, id)
        move_obj = self.pool.get('stock.move')
        procurement_obj = self.pool.get('procurement.order')
        wf_service = netsvc.LocalService("workflow")
        if picking.state != 'done':
            wf_service.trg_validate(uid, 'stock.picking', id, 'button_cancel', cr)
        else:
            data = {
                'state': 'confirmed',
                'origin': picking.origin + '-REV',
                'type': picking.type,
                'move_type': picking.move_type,
                'address_id': picking.address_id.id,
                'auto_picking': picking.auto_picking,
            }

            reverse_picking_id = self.create(cr, uid, data)
            reverse_picking = self.browse(cr, uid, reverse_picking_id)
            move_ids = []
            proc_ids = []
            for move in picking.move_lines:
                move_id = move_obj.create(
                    cr, 
                    uid, 
                    {
                        'name':move.name,
                        'product_id': move.product_id.id,
                        'product_qty': move.product_qty,
                        'product_uom': move.product_uom.id,
                        'product_uos_qty': move.product_uos_qty,
                        'product_uos': move.product_uos.id,
                        'date_expected': move.date_expected,
                        'address_id': move.address_id.id,
                        'company_id': move.company_id.id,
                        'origin': move.origin,
                        'partner_id': move.partner_id.id,
                        
                        'picking_id': reverse_picking_id,
                        'state': reversal_state,
                        'location_id': move.location_dest_id.id,
                        'location_dest_id': move.location_id.id
                    })
                move_ids.append(move_id)
                for proc in move.procurements:
                    proc_id = procurement_obj.create(
                        cr, 
                        uid, 
                        {
                            'name': proc.name,
                            'origin': proc.origin,
                            'date_planned': proc.date_planned,
                            'product_id': proc.product_id.id,
                            'product_qty': proc.product_qty,
                            'product_uom': proc.product_uom.id,
                            'product_uos_qty': proc.product_uos_qty,
                            'product_uos': proc.product_uos.id,
                            'location_id': move.location_dest_id.id, #reversed
                            'procure_method': proc.procure_method,
                            'move_id': move_id,
                        })
                    wf_service.trg_validate(
                        uid, 
                        'mrp.procurement', 
                        proc_id, 
                        'button_confirm', 
                        cr)
            move_obj.check_assign(cr, uid, move_ids)
            wf_service.trg_validate(uid, 'stock.picking', reverse_picking_id, 'button_confirm', cr)

stock_picking()
