from osv import fields, osv
from datetime import date
from business_date import add_business_days, sub_business_days
from mx import DateTime

class procurement_order(osv.osv):
    """
    Hook into the procurement.order handling so that we use
    the date calculation which respects weekdays
    """
    _inherit = "procurement.order"

    def action_confirm(self, cr, uid, ids, context={}):
        result = super(mrp_procurement,self).action_confirm(cr,uid,ids,context)
        for procurement in self.browse(cr, uid, ids):
            # Does this procurement require a date_planned?
            # We figure that out by tracing back to the sales order line.
            if (not procurement.move_id.sale_line_id):
                 continue
            
            date_planned = add_business_days(
                fields.date.context_today(self, cr, uid, context=context), 
                procurement.move_id.sale_line_id.delay)
            self.write(
                cr, 
                uid, 
                procurement.id, 
                { 'date_planned': date_planned })

        return result

    def action_produce_assign_product (self, cr, uid, ids, context={}):
        produce_id = super(mrp_procurement,self).action_produce_assign_product(
            cr,
            uid,
            ids,
            context)
        if produce_id == False: 
            return False

        company = self.pool.get('res.users').browse(cr, uid, uid, context).company_id
        for procurement in self.browse(cr, uid, ids):
            res_id   = procurement.move_id.id
            loc_id   = procurement.location_id.id
            date_planned  = sub_business_days( 
                          DateTime.strptime(procurement.date_planned, '%Y-%m-%d %H:%M:%S'),
                          procurement.product_id.product_tmpl_id.produce_delay or 0.0)
            date_planned  = sub_business_days( 
                          date_planned, 
                          company.manufacturing_lead)
            self.pool.get('mrp.production').write(
                  cr, 
                  uid, 
                  produce_id,
                  { 'date_planned': date_planned.strftime('%Y-%m-%d %H:%M:%S') })

        return produce_id

mrp_procurement()



