from business_date import add_business_days
from datetime import date, datetime
import openerp.tools as tools
from osv import osv, fields

class related_boolean(fields.related):

    def _fnct_read(self, obj, cr, uid, ids, field_name, args, context=None):
        """ This just makes sure that any returned result that has a
            null value has a "false" value.
        """
        result = super(related_boolean,self)._fnct_read(obj, cr, uid, ids, field_name, args, context)
        for k,v in result.items():
            if v == None:
                result[k] = v
        return result



class stock_move(osv.osv):
    _inherit = 'stock.move'
    _order = 'promise_date, partner_id, id'

    def _note_shortened (self, cr, uid, ids, field_name, arg, context):
        """ Return the first line of the note field for display in the tree.
        
        Also combines the internal and external notes, and truncates the whole
        thing if it's too long.
        """
        PRODUCT_NOTES_TREE_VIEW_MAX_LENGTH = 20
        result = {}
        for move in self.browse(cr, uid, ids, context=context):
            if field_name == 'move_notes_shortened':
                trimmed = (move.note or '').partition('\n')[0]
            else:
                trimmed = (move.sale_notes or '').partition('\n')[0]
            
            if ( len(trimmed) <= PRODUCT_NOTES_TREE_VIEW_MAX_LENGTH ): 
                result[move.id] = trimmed 
            else:
                result[move.id] = trimmed[:PRODUCT_NOTES_TREE_VIEW_MAX_LENGTH] + "..."
        return result
    
    def search(
        self, 
        cr, 
        user, 
        args, 
        offset=0, 
        limit=None, 
        order=None,
        context=None, 
        count=False):

        if not order:
            if len(args) == 1 and args[0][0] == 'picking_id':
                # Query for a single picking list uses a different default order
                order = 'grouping, promise_date, id'
            else:
                # Weird bug where ordering by partner_id (actually name)
                # only works with explicit order, not default.
                order = self._order        
        else:
          if order == 'sanity_checked':
              order = 'sanity_checked, ' + self._order
          elif order == 'sanity_checked desc':
              order = 'sanity_checked desc, ' + self._order


        ids = super(stock_move, self).search(
            cr, 
            user, 
            args, 
            offset, 
            limit, 
            order, 
            context, 
            count)
        return ids
    
    def _read_flat(
        self, 
        cr, 
        user, 
        ids, 
        fields_to_read, 
        context=None, 
        load='_classic_read'):
        rows = super(stock_move, self)._read_flat(
            cr, 
            user, 
            ids, 
            fields_to_read, 
            context, 
            load)
        if fields_to_read == ['picking_id']:
            # Stock moves within a stock picking should be sorted by grouping.
            # ids were sorted by grouping in the search() method, so sort
            # these rows in the same order they appear in ids.
            rows.sort(key= lambda row: ids.index(row['id']))
        return rows

    def _get_sanity_stock_moves_to_update(picking_obj, cr, uid, picking_ids, context=None):
        """ Get all the stock moves related to the picking object """
        moves = set()
        move_obj = picking_obj.pool.get('stock.move')
        for pick in picking_obj.browse(cr,uid,picking_ids,context):
            moves.update(
                move_obj.search(cr,uid,[('picking_id','in',picking_ids)])
            )
        return moves
 
    _columns = {
        'picking_date': fields.related(
            'picking_id',
            'date',
            type='date',
            string='Date Ordered',
            readonly=True,
            help='Date from the attached picking list.'),
        'picking_date_cleared': fields.related(
            'picking_id',
            'date_cleared',
            type='date',
            readonly=True,
            string='Cleared to Ship',
            help='Date from the attached picking list.'),
        'picking_date_done': fields.related(
            'picking_id',
            'date_done',
            type='date',
            readonly=True,
            string='Date Done',
            help='Date from the attached picking list.'),
        'origin': fields.related(
            'picking_id',
            'origin',
            type='char',
            size=64,
            readonly=True,
            string='Origin Reference',
            help='Origin from the attached picking list.'),
        # Replace the address_id field with a copy from the packing list.
        # Store the field, because it's used by the report_stock_move view.
        'address_id': fields.related(
            'picking_id',
            'address_id',
            type='many2one',
            readonly=True,
            string='Partner',
            store=True,
            relation='res.partner.address'),
        'backorder_id': fields.related(
            'picking_id',
            'backorder_id',
            type='many2one',
            readonly=True,
            string='Back Order',
            relation='stock.picking'),
        'client_order_ref': fields.related( # The Purchase Order #
            'sale_line_id',
            'order_id',
            'client_order_ref',
            type = 'char',
            size=64,
            readonly=True,
            string='Customer Ref'),
        'carrier_id': fields.related(
            'picking_id',
            'carrier_id',
            type='many2one',
            readonly=True,
            string='Carrier',
            relation='delivery.carrier'),
        'sanity_checked': related_boolean(
            'picking_id',
            'sanity_checked',
            type = 'boolean',
            string = 'Checked',
            store = {
              'stock.picking': (
                _get_sanity_stock_moves_to_update,
                ['sanity_checked'],
                10
              ),
            },
            readonly = True
        ),
        'promise_date':fields.date(
            'Promised',
            help='The date this line item was promised to the customer'),
        'sale_notes' : fields.related(
            'sale_line_id',
            'notes',
            type='text',
            string='Sales Order Notes'),
        'sale_notes_shortened': fields.function(
            _note_shortened,
            type = 'char',
            method = True,
            string = 'External Note'),
        'move_notes_shortened': fields.function(
            _note_shortened,
            type = 'char',
            method = True,
            string = 'Internal Note'),
        'product_code': fields.related(
            'product_id',
            'code',
            type='char',
            size=64,
            string='Product'),
        'product_revision': fields.related(
            'product_id',
            'revision',
            type='integer',
            string='Product Revision'),
        'location_dest_usage': fields.related(
            'location_dest_id',
            'usage',
            type='char',
            string='Dest. Location Type',
            readonly=True,
        ),
    }

    _default = {
      'sanity_checked': False,
    }
    
    def action_confirm(self, cr, uid, ids, context={}):
        super(stock_move,self).action_confirm(cr,uid,ids,context)
        if context is None:
            user = self.pool.get('res.users').browse(cr, uid, uid)
            context = {'tz': user.context_tz}
            
        for move in self.browse(cr, uid, ids):
            # Does this move require a promise_date?
            # We figure that out by tracing back to the sales order line.
            if (not move.sale_line_id):
                 continue
            
            today = datetime.strptime(
                fields.date.context_today(self, cr, uid, context=context),
                tools.DEFAULT_SERVER_DATE_FORMAT)
            now = datetime.now().replace(microsecond=0)

            promise_date = add_business_days(
                today, 
                move.sale_line_id.delay)
            date_expected = add_business_days(
                now,
                move.sale_line_id.delay)
            self.write(
                cr, 
                uid, 
                move.id, 
                {
                    'promise_date': promise_date,
                    'date_expected': date_expected
                })

        return []
stock_move()

class stock_picking(osv.osv):
    '''
    Include the Client Order Reference field (aka. PO#) in the 
    available packing screens
    '''
    _inherit = 'stock.picking'
    _order = 'promise_date, id'
    
    def _min_promise_date(self, cr, uid, ids, field_name, arg, context):
        promise_dates = {}
        for picking in self.browse(cr, uid, ids):
            first_promise_date = None
            for move in picking.move_lines:
                if not first_promise_date:
                    first_promise_date = move.promise_date
                elif move.promise_date and move.promise_date < first_promise_date:
                    first_promise_date = move.promise_date
            promise_dates[picking.id] = first_promise_date
        return promise_dates
    
    def _get_move_pickings(self, cr, uid, ids, context=None):
        moves = self.pool.get('stock.move').browse(cr, uid, ids, context)
        pickings = set()
        for move in moves:
            if move.picking_id:
                pickings.add(move.picking_id.id)
        return pickings
        
    _columns = {
        'client_order_ref': fields.related( # The Purchase Order #
            'sale_id',
            'client_order_ref',
            type = 'char',
            size = 64,
            readonly=True,
            string='Customer Ref'),
        'promise_date': fields.function(
            _min_promise_date,
            type = 'date',
            method = True,
            string = 'Promise Date',
            help = 'Earliest promise date from the stock moves',
            store = {
                'stock.move': (
                    _get_move_pickings,
                    ['promise_date'],
                    10)}),
        'sanity_checked': fields.boolean(
                'Sanity Checked',
                help = 'Has production checked that all items in the packing '
                    'list make sense together?')
    }
stock_picking()
