import time
import datetime
from osv import fields, osv
from tools.misc import DEFAULT_SERVER_DATETIME_FORMAT
from tools.translate import _

class reschedule_picking(osv.osv_memory):
    """Updates the date fields on all stock moves in a picking."""
    _name = 'reschedule.picking'
    _description = "Reschedule Picking"

    _columns = {
        'picking_id': fields.many2one('stock.picking', 'Picking'),
        'date_expected': fields.datetime(
            'Scheduled Date', 
            required=True, 
            help="Scheduled date for the processing of this picking"),
        'promise_date':fields.date(
            'Promised',
            help='The date this picking was promised to the customer'),
    }

    def default_get(self, cr, uid, fields, context=None):
        if context is None: 
            context = {}
        res = super(reschedule_picking, self).default_get(cr, 
                                                          uid, 
                                                          fields, 
                                                          context)
        
        picking_ids = context.get('active_ids', [])
        if len(picking_ids) != 1:
            return res
        if 'date_expected' in fields or 'promise_date' in fields:
            picking = self.pool.get('stock.picking').browse(cr, 
                                                            uid, 
                                                            picking_ids[0], 
                                                            context)
        if 'picking_id' in fields:
            res.update(picking_id=picking_ids[0])
        if 'date_expected' in fields:
            res.update(date_expected=min([move.date_expected
                                          for move in picking.move_lines]))
        if 'promise_date' in fields:
            res.update(promise_date=min([move.promise_date
                                         for move in picking.move_lines]))
        return res

    def do_reschedule(self, cr, uid, ids, context=None):
        reschedule = self.browse(cr, uid, ids[0], context)
        
        move_obj = self.pool.get('stock.move')
        for move in reschedule.picking_id.move_lines:
            if move.state in ('done', 'cancel'):
                raise osv.except_osv(
                    _('Error!'), 
                    _('Cannot reschedule a picking with moves in the %s state.') % 
                        move.state)
        
            move_obj.write(cr, 
                           uid, 
                           move.id, 
                           {'date_expected': reschedule.date_expected,
                            'date': reschedule.date_expected,
                            'promise_date': reschedule.promise_date},
                           context=context)
        
        # Trigger a recalculation of the min_date field.
        self.pool.get('stock.picking').write(cr,
                                             uid,
                                             reschedule.picking_id.id,
                                             {'min_date': False},
                                             context=context)
        return {}
