import time
import datetime
from osv import fields, osv
from tools.misc import DEFAULT_SERVER_DATETIME_FORMAT
from tools.translate import _

class reschedule_purchase(osv.osv_memory):
    """Updates the date fields on all purchase order lines."""
    _name = 'reschedule.purchase'
    _description = "Reschedule Purchase"

    _columns = {
        'order_id': fields.many2one('purchase.order', 'Purchase Order'),
        'date_planned': fields.date(
            'Scheduled Date', 
            required=True, 
            help="Scheduled date for the processing of this purchase order"),
    }

    def default_get(self, cr, uid, fields, context=None):
        if context is None: 
            context = {}
        res = super(reschedule_purchase, self).default_get(cr, 
                                                           uid, 
                                                           fields, 
                                                           context)
        
        order_ids = context.get('active_ids', [])
        if len(order_ids) != 1:
            return res
        if 'date_planned' in fields:
            order = self.pool.get('purchase.order').browse(cr, 
                                                           uid, 
                                                           order_ids[0], 
                                                           context)
        if 'order_id' in fields:
            res.update(order_id=order_ids[0])
        if 'date_planned' in fields:
            res.update(date_planned=min([line.date_planned
                                         for line in order.order_line]))
        return res

    def do_reschedule(self, cr, uid, ids, context=None):
        reschedule = self.browse(cr, uid, ids[0], context)
        order = reschedule.order_id
        if order.state in ('done', 'cancel'):
            raise osv.except_osv(
                _('Error!'), 
                _('Cannot reschedule a purchase order in the %s state.') % 
                    order.state)
        
        line_obj = self.pool.get('purchase.order.line')
        for line in order.order_line:
            if line.state in ('done', 'cancel'):
                raise osv.except_osv(
                    _('Error!'), 
                    _('Cannot reschedule a purchase order with lines in the %s state.') % 
                        line.state)
        
            line_obj.write(cr, 
                           uid, 
                           line.id, 
                           {'date_planned': reschedule.date_planned},
                           context=context)
        
        return {}
