# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2010 Zaber Inc. All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


{
    "name" : "Zaber Specific Customizations to interfaces",
    "version" : "0.2",
    "author" : "Zaber Inc (Aki Mimoto)",
    'complexity': "easy",
    "website" : "http://www.zaber.com",
    "category" : "Generic Modules/Inventory Control",
    "depends" : ["account", 
                 "account_voucher",
                 "combined_lines",
                 "crm_helpdesk",
                 "hr",
                 "hr_attendance",
                 "hr_timesheet_sheet", 
                 "mrp", 
                 "partner_function",
                 "product", 
                 "production_qty",
                 "project_timesheet",
                 "sale", 
                 "purchase",
                 "procurement"],
    "description": """
      Customizations of the interface and modules that are specific to Zaber's needs. This module is a collection of small interface tweaks that otherwise might of just ended up in the core modules.

      On Company View form:

        - Add and allow editing of new CAD/USD Wire transfer information
          that will be used in invoice printouts.

      On CRM Cases form:

        - Disable "Send Partner & Historize" button 
        - Remove "Send Reminder" button next to partner.
        - Update the "Send Reminder" to "Send Reminder to Responsible" to reduce confusion
        - Add a line (made of underscores) below each message entry to show different entries

      On CRM Cases View tree:

        - Display the Partner Contact's name beside Partner Name
        - Display allow searching on the Partner Contact's Name by default
        - Ignore what the current Partner Email might be, overwrite when a new partner or contact is chosen
        - Remove Deadline
        - Remove Planned Revenue
        - Remove Planned Cost
        - Replace Date with Last Action Date
        - Make Last Action Date a searchable field
        - Limit Parter Contact Name to 30 characters

      On Employees tree:
        - Add Code field.

      On Employees form:
        - Add Code field.
        
      On Invoices View form:

        - Changing the partner will update the Currency field to the currency of the default 
          purchase or sales price list of the partner.

      On Partner View form:

        - Hide the Mass Mail and Send SMS wizard buttons

      On Partner View tree:

        - Show "Sale Pricelist" at the very end of the column lists

      On Partner Address View tree:

        - Add Phone and Email columns to list view and change column order to 
          Partner, Contact Name, Function, Phone, Email, City, Country, Zip

      On Packing List form:

        - Change the name of the "Cancel" button to something a bit more 
          intuitive (so people don't mistake it as being the "cancel to 
          leave screen" button)

      On Purchase Order tree view:
        - Order by Received state then by planned date

      On Production Order form:

        - Change the name of the "Cancel" button to something a bit more 
          intuitive (so people don't mistake it as being the "cancel to 
          leave screen" button)

      On Production Order tree:

        - Move state column to far left

      On Product View form:

        - Include "Bill of Materials Components" button on right hand side

      On Product View tree:

        - Show the default Supplier Name and make it searchable
        - Change column order to: status, code, revision, name, supplier name, 
          note, days until reorder, real stock, virtual stock, sales capacity, 
          committed stock, list price, cost price, default uofm, category


      Product Categories:

        - Add the fields
          - description
          - show
          - producer
          - net_cost
          - harmonized_code
          - ship_description
          - preference_criterion
          - origin_country_id
 
      On Purchase Order form:

        - Add an "Attention To:" field for POs

      On Purchase Order Line View tree:

        - Add searchable column product default code
        - Add searchable column date planned
        - Add searchable column default product code
        - Add searchable column supplier's name
        - Add searchable column supplier's partnumber

      On Purchase Order form:

        - Add "Revision" to the columns shown for selected components
        - Include Drawing Number on PO line item

      On Purchase Order table:

        - Ensure that purchase.order.name is read-write (purchase_manual) makes it read-only and names it weird

      On Sale Order tree view:

        - Remove Partner's Shipping Address

      On BoM Components Tree View

        - Add the Parent BoM's name
        - Add the Cost Price of component
        - Add the Sequence value

      On Confirmed or Available Packing Items View

        - Change the button name from "Cancel" to "Cancel Item"

      In Financial Management

        - Add Account Balances. Creates a screen with all bank balances

      Behaviour changes

        - Copy over *only* the sales order notes when creating an invoice from Outgoing Products screen
        - Delete cached invoice pdf when invoice cancelled. Allows users to make amendments to invoices.
        - Invoices will be sorted in inverse chronological order of creation
        - Searching for a product will no longer follow OpenERP's rules. It will substring match both the default code and name fields.

      Security tweaks

        - Allow system users to make changes to the outgoing mail server (default is to allow only the administrator)

    """,
    'init_xml': [],
    'images' : [],
    'update_xml': [
        'security/account_security.xml',
        'security/ir.model.access.csv',
        'wizard/crm_add_history_view.xml',
        'wizard/make_procurement_view.xml',
        'account_view.xml',
        'crm_helpdesk_view.xml',
        'hr_timesheet_sheet_view.xml',
        'hr_view.xml',
        'mail_message_view.xml',
        'mrp_workflow.xml',
        'mrp_view.xml',
        'partner_view.xml',
        'product_view.xml',
        'project_view.xml',
        'res_company_view.xml',
        'sale_view.xml',
        'purchase_view.xml',
        'stock_view.xml',
    ],
    'demo_xml': [],
    'installable': True,
    'auto_install': False,
    'certificate': '',
}
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
