from __future__ import absolute_import
from datetime import date, datetime, timedelta
from dateutil.relativedelta import relativedelta
from osv import fields,osv,orm
from tools import config
from tools.translate import _
import netsvc
import account
import account.account_invoice


class account_account(osv.osv):
    ''' Make it possible for admin to see a list of all the financial
        accounts but make sure they can't seen the individual account
        entry line moves
    '''

    _inherit = 'account.account'

    def _ytd_balance (self, cr, uid, ids, field_names, arg=None, context=None, \
                  query='', query_params=()):
        ''' compute the balances of the accounts provided but only for the
            current year (note: year, not fiscal year)
        '''

        mapping = {
            'ytd_debit': 'debit',
            'ytd_credit': 'credit',
            'ytd_balance': 'balance',
        }

        remapped_field_names = map(lambda x: mapping.get(x,x), field_names)

        if query:
            query += " and "

        # figure out what date range we're wanting to search
        date_today    = date.today()
        date_jan1st   = date(date_today.year,1,1)
        date_lastyear = date(date_today.year-1,1,1)
        date_range_end = date_today + timedelta(1)

        # (just if it's dec 31st)
        if date_range_end.year != date_today.year:
            date_range_end = date_today

        # Reject the special case of opening balances from the previous year
        current_fiscal_year = self.pool.get('account.fiscalyear').search(cr,uid,[
                                ('date_start','>=',date_jan1st),
                                ('date_stop','>=',date_today),
                              ])[0]
        deny_period_id = self.pool.get('account.period').search(cr,uid,[
                                ('fiscalyear_id','=',current_fiscal_year),
                                ('special','=','t')
                              ])[0]

        query += "date >= '"+date_lastyear.isoformat()+"' and "+\
                 "date <= '"+date_range_end.isoformat()+"' and "+\
                 "period_id != "+str(deny_period_id)

        res = self.__compute(
                            cr, uid, ids, remapped_field_names, arg, context, query, query_params
                        )

        # rekey everything (since we get things back as "debit", "credit", "balance",
        # we need to prefix them with "ytd_" to be correct
        for row in res.values():
            # bug in __compute means that some entries in res aren't rows.
            if isinstance(row, dict):
                for new_field, old_field in mapping.iteritems():
                    if old_field in row:
                        row[new_field] = row[old_field]
                        del row[old_field]

        return res

    def _currency_balance(self, cr, uid, ids, field_names, arg=None, context=None, \
                  query='', query_params=()):
        """ compute the balance, debit and/or credit for the provided
        account ids in the alternate currency

        Arguments:
        `ids`: account ids
        `field_names`: the fields to compute (a list of any of
                       'balance', 'debit' and 'credit')
        `arg`: unused fields.function stuff
        `query`: additional query filter (as a string)
        `query_params`: parameters for the provided query string
                        (__compute will handle their escaping) as a
                        tuple
        """
        #get all the necessary accounts
        children_and_consolidated = self._get_children_and_consol(
            cr, uid, ids, context=context)
        #compute for each account the balance/debit/credit from the move lines
        accounts = {}
        if children_and_consolidated:

            # figure out what date range we're wanting to search
            date_today    = date.today()
            date_jan1st   = date(date_today.year,1,1)
            date_lastyear = date(date_today.year-1,1,1)
            date_range_end = date_today + timedelta(1)

            # (just if it's dec 31st)
            if date_range_end.year != date_today.year:
                date_range_end = date_today

            fiscal_year_ids = self.pool.get('account.fiscalyear').search(cr,uid,[
                                ('date_start','>=',date_lastyear),
                                ])

            context['fiscalyear'] = ",".join(map(str, fiscal_year_ids))
            aml_query = self.pool.get('account.move.line')._query_get(cr, uid, context=context)

            wheres = [ "", "l.date <= '"+date_range_end.isoformat()+"'" ]

            # Reject the special case of opening balances from the previous year
            current_fiscal_year = self.pool.get('account.fiscalyear').search(cr,uid,[
                                    ('date_start','>=',date_jan1st),
                                    ('date_stop','>=',date_today),
                                  ])[0]
            deny_period_id = self.pool.get('account.period').search(cr,uid,[
                                    ('fiscalyear_id','=',current_fiscal_year),
                                    ('special','=','t')
                                  ])[0]
            wheres.append('l.period_id != %d'%deny_period_id) 

            if query.strip():
                wheres.append(query.strip())
            if aml_query.strip():
                wheres.append(aml_query.strip())
            filters = " AND ".join(wheres)
            self.logger.notifyChannel('addons.'+self._name, netsvc.LOG_DEBUG,
                                      'Filters: %s'%filters)
            # IN might not work ideally in case there are too many
            # children_and_consolidated, in that case join on a
            # values() e.g.:
            # SELECT l.account_id as id FROM account_move_line l
            # INNER JOIN (VALUES (id1), (id2), (id3), ...) AS tmp (id)
            # ON l.account_id = tmp.id
            # or make _get_children_and_consol return a query and join on that
            request = ("""SELECT l.account_id as id, c.name as currency_name,
                          COALESCE(SUM(
                          case
                          when l.credit = 0 then     l.amount_currency
                          when l.credit is null then l.amount_currency
                          else l.amount_currency
                          end),0) as alternate_currency_balance,
                          COALESCE(SUM(l.debit),0) - COALESCE(SUM(l.credit), 0) as balance
                         FROM account_move_line l
                       LEFT JOIN res_currency    as c on ( l.currency_id = c.id )
                       WHERE l.account_id IN %s
                          {filters}
                       GROUP BY l.account_id,c.name
                       """.format(filters=filters))

            params = (tuple(children_and_consolidated),) + query_params
            cr.execute(request, params)
            self.logger.notifyChannel('addons.'+self._name, netsvc.LOG_DEBUG,
                                      'Status: %s'%cr.statusmessage)

            # Accounts will partition the transactions by the different currencies.
            for res in cr.dictfetchall():
                currency_name = res['currency_name'] or ''
                if res['id'] not in accounts:
                    accounts[res['id']] = {}
                accounts[res['id']][currency_name] = res

        # consolidate accounts with direct children
        children_and_consolidated.reverse()
        brs = list(self.browse(cr, uid, children_and_consolidated, context=context))
        currency_sums = {}

        # This does multiple passes over an array to see if an account ID can 
        # be calculated fully. While the solution here can have a worst case 
        # scenario of O(n!), it appears they try and optimize it with the 
        # .reverse() above
        while brs:
            current = brs[0]

            # Do we have all immediate descendants fully calculated?
            # If not, we'll just skip it
            can_compute = True
            for child in current.child_id:
                if child.id not in currency_sums:
                    can_compute = False
                    try:
                        brs.insert(0, brs.pop(brs.index(child)))
                    except ValueError:
                        brs.insert(0, child)

            # Nope, an immediate child still requires calculation
            if not can_compute:
                continue

            brs.pop(0)

            # Flag this account as being "calculated/examined"
            currency_sums.setdefault(current.id, {})

            # First add the account sums directly associated with this accont
            if current.id in accounts:
                for currency_name, value in accounts[current.id].iteritems():
                    if currency_name == "":
                        currency_name = current.company_currency_id.name
                        currency_sums[current.id][currency_name] = value['balance']
                    else:
                        currency_sums[current.id][currency_name] = value['alternate_currency_balance']

            # Then add all the child accounts only if there are children
            if not current.child_id:
                continue

            for child in current.child_id:
                if child.id not in currency_sums:
                    continue

                for currency_name, value in currency_sums[child.id].iteritems():
                    currency_sums[current.id].setdefault(currency_name,0.0)
                    currency_sums[current.id][currency_name] += currency_sums[child.id][currency_name]

        sums = {}
        res = {}
        null_result = dict((fn, 0.0) for fn in field_names)

        lang = context.get('lang', 'en_US') or 'en_US'
        lang_obj_ids = self.pool.get('res.lang').search(cr, uid,[('code','=',lang)])
        if not lang_obj_ids:
            raise Exception(_('Language with code "%s" is not defined in your system !\nDefine it through the Administration menu.') % (lang,))
        lang_obj = self.pool.get('res.lang').browse(cr, uid, lang_obj_ids[0])
        translation_obj = self.pool.get('ir.translation')

        for id in ids:
            if id in currency_sums:
                currency_lines = []
                for currency_name, currency_value in currency_sums[id].iteritems():
                    if currency_value == 0: continue
                    # currency_lines.append(str(currency_value) + " " + currency_name)
                    currency_lines.append(
                        lang_obj.format('%.'+str(2) + "f", currency_value, grouping=True, monetary=True) 
                        + " " + currency_name
                        )
                res[id] = " ".join(currency_lines)
            else:
                res[id] = 0.0
        return res

    _columns = {

        'alternate_currency_balance': fields.function(
                        _currency_balance, 
                        type='text',
                        method=True, 
                        string='Alternate Currency Balance',
                        ),

        'ytd_debit': fields.function(
                        _ytd_balance, 
                        digits=(16, int(config['price_accuracy'])), 
                        method=True, 
                        string='Debit',
                        multi='balance',
                        ),

        'ytd_credit': fields.function(
                        _ytd_balance, 
                        digits=(16, int(config['price_accuracy'])), 
                        method=True, 
                        string='Credit',
                        multi='balance',
                        ),

        'ytd_balance': fields.function(
                        _ytd_balance, 
                        digits=(16, int(config['price_accuracy'])), 
                        method=True, 
                        string='Balance',
                        multi='balance',
                        ),
        
        'is_sensitive': fields.boolean(
                        'Is Sensitive',
                        help='Sensitive account entries can ' +
                        'only be seen by accounting managers.'),

        'is_account_balances_visible': fields.boolean(
                        'Balance Visibility',
                        help='Let the account balance be ' +
                        'visible in Account Balances report.')

    }

    _defaults = {
      'is_account_balances_visible': True
    }

account_account()

class account_fiscalyear(osv.osv):
    _inherit = "account.fiscalyear"

    def create_period(self, cr, uid, ids, context=None, interval=1):
        period_obj = self.pool.get('account.period')
        for fy in self.browse(cr, uid, ids, context=context):
            ds = datetime.strptime(fy.date_start, '%Y-%m-%d')
            period_obj.create(cr, uid, {
                    'name':  "%s %s" % (_('Opening Period'), ds.strftime('%Y')),
                    'code': ds.strftime('%Y/00'),
                    'date_start': ds,
                    'date_stop': ds,
                    'special': True,
                    'fiscalyear_id': fy.id,
                })
            while ds.strftime('%Y-%m-%d') < fy.date_stop:
                de = ds + relativedelta(months=interval, days=-1)

                if de.strftime('%Y-%m-%d') > fy.date_stop:
                    de = datetime.strptime(fy.date_stop, '%Y-%m-%d')

                period_obj.create(cr, uid, {
                    'name': ds.strftime('%Y/%m'),
                    'code': ds.strftime('%Y/%m'),
                    'date_start': ds.strftime('%Y-%m-%d'),
                    'date_stop': de.strftime('%Y-%m-%d'),
                    'fiscalyear_id': fy.id,
                })
                ds = ds + relativedelta(months=interval)
        return True
account_fiscalyear()

class account_invoice(osv.osv):
    _inherit = 'account.invoice'

    _columns = {
        'journal_ids': fields.function(
                            account.account_invoice.account_invoice._compute_lines, 
                            relation='account.move.line', 
                            type="many2many", 
                            string='Journal Entries'
                        ),
    }
    
    def invoice_pay_customer(self, cr, uid, ids, context=None):
        """ Populate the default Payment ref value with our internal invoice 
        number and don't put customer invoice number in the memo field 
        (default_name)."""
        data = super(account_invoice, self).invoice_pay_customer(cr,
                                                                 uid,
                                                                 ids,
                                                                 context=context)
        if data and ids:
            inv = self.browse(cr, uid, ids[0], context=context)
            data['context']['default_reference'] = (
                inv.number
                if inv.type in ('out_invoice', 'out_refund')
                else inv.name)
            data['context']['default_name'] = ''
            data['context']['default_journal_id'] = False
            data['context']['default_date'] = False
        return data
        
account_invoice()

class account_move(osv.osv):
    _inherit = 'account.move'
    
    def create(self, cr, uid, vals, context=None):
        if context == None or not context.get('copy'):
          move_lines = vals.get('line_id')
          if move_lines:
              move_line = move_lines[0]
              if len(move_line) > 2:
                  fields = move_line[2]
                  invoice_number = fields.get('ref')
                  if invoice_number:
                      vals['ref'] = invoice_number

        return super(account_move, self).create(cr, uid, vals, context)
account_move()

class account_move_line(osv.osv):
    _inherit = 'account.move.line'
    
    def onchange_analytic_account_id(self, 
                                     cr, 
                                     uid, 
                                     ids, 
                                     analytic_account_id, 
                                     analytic_lines,
                                     debit,
                                     credit):
        warning = {}
        if analytic_account_id:
            analytic_line_obj = self.pool.get('account.analytic.line')
            analytic_line_count = analytic_line_total = 0
            for action, id, fields in analytic_lines:
                if fields and 'amount' in fields:
                    analytic_line_count += 1
                    analytic_line_total += fields['amount']
                elif 1 <= action <= 4:
                    analytic_line = analytic_line_obj.browse(cr, uid, id)
                    analytic_line_count += 1
                    analytic_line_total += analytic_line.amount
                    
            if (analytic_line_count > 1 or
                (analytic_line_count == 1 and
                 analytic_line_total != (credit - debit))):
                
                warning['title'] = 'Replacing Analytic Entries'
                warning['message'] = (
                    "This entry line has custom analytic lines. Setting an " +
                    "analytic account will replace them.")
            
        return {'warning': warning}
    
    def create(self, cr, uid, vals, context=None):
        """ In version 6, they made account_move_line.ref a related field that
        just copies account_move.ref, but it's buggy. We decided to unlink
        the two ref fields, but that means that account_move_line.ref often gets
        left blank. This code will copy the ref from account_move if there is
        no value for account_move_line.ref."""
        move_id = vals.get('move_id')
        ref = vals.get('ref')
        if move_id and not ref:
            move = self.pool.get('account.move').browse(cr, 
                                                        uid, 
                                                        move_id, 
                                                        context)
            vals['ref'] = move.ref
        return super(account_move_line, self).create(cr, uid, vals, context)

    _columns = {
        'name': fields.char('Name', size=255, required=True), # widen original
        'ref': fields.char('Ref.', size=64), # remove related field
    }


class account_voucher(osv.osv):
    _inherit = 'account.voucher'
    _defaults = {
        'journal_id':None,
    }

