from osv import osv

class account_invoice(osv.osv):
    _inherit = "account.invoice"

    # So that we see invoices sorted in reverse chronological order of creation
    _order = "create_date desc, date_invoice desc, id desc"

    def onchange_partner_id(self, cr, uid, ids, type, partner_id,
            date_invoice=False, payment_term=False, partner_bank_id=False, 
            company_id=False):
        ''' We also want to include the default currency for the particular
            partner
        '''

        res = super(account_invoice,self).onchange_partner_id(cr, 
                                                              uid, 
                                                              ids, 
                                                              type, 
                                                              partner_id,
                                                              date_invoice, 
                                                              payment_term, 
                                                              partner_bank_id,
                                                              company_id)

        if not partner_id:
            return res

        partner = self.pool.get('res.partner').browse(cr, uid, partner_id)

        if type in ('out_invoice', 'out_refund'):
            if partner.property_product_pricelist:
                res['value']['currency_id'] = (
                    partner.property_product_pricelist.currency_id.id)
        else:
            if partner.property_product_pricelist_purchase:
                res['value']['currency_id'] = (
                    partner.property_product_pricelist_purchase.currency_id.id)

        return res

    def action_cancel(self, cr, uid, ids, *args):
        ''' When a user cancels an invoice, remove the previously generated pdf 
        invoice from the attachment table. This will allow users to make 
        amendments that can then be viewed after the change.
        '''

        res = super(account_invoice, self).action_cancel(cr, uid, ids, args)


        # Don't bother trying if there's no work to do
        if len(ids) <= 0:
            return res

        # Find all the attachments associated with these invoices
        # Searching for attachments on a list of objects doesn't work in v6.1.
        ia_obj = self.pool.get('ir.attachment')
        attachment_ids = []
        for id in ids:
            attachment_ids.extend(ia_obj.search(
                cr, 
                uid, 
                [('res_model', '=', 'account.invoice'),
                 ( 'res_id', '=', id )]))

        # Now nuke 'em if it's required
        if not attachment_ids:  
            return res
        ia_obj.unlink(cr,uid,attachment_ids)

        return res

account_invoice()

