import datetime
from osv import fields,osv,orm
from tools import config

class mrp_bom(osv.osv):
    ''' 
    Create a link to the product's Cost Price. This will allow the display of the 
    product's Cost Price in the BoM Display
    '''
    _inherit = 'mrp.bom'

    _columns = {'code': fields.char('Reference', size=64), #increase width
                'standard_price': fields.related('product_id',
                                                 'product_tmpl_id',
                                                 'standard_price',
                                                 type='float',
                                                 string='Cost Price'),
                'categ_id': fields.related('product_id',
                                           'product_tmpl_id',
                                           'categ_id',
                                           type='many2one',
                                           relation='product.category',
                                           string='Product Category'),
                'parent_product_id': fields.related('bom_id',
                                                    'product_id',
                                                    type='many2one',
                                                    readonly=True,
                                                    string='Parent Product',
                                                    relation='product.product'),
                'parent_sequence': fields.related('bom_id',
                                                  'sequence',
                                                  type='integer',
                                                  readonly=True,
                                                  string='Parent Sequence'),
                'autocomplete': fields.boolean(
                    'Autocomplete',
                    help="Will production orders based on this BoM be " +
                        "automatically completed as soon as the raw " +
                        "materials are available?"),
    }

mrp_bom()

class mrp_production(osv.osv):
    _inherit = 'mrp.production'
    
    _columns = {'autocomplete': fields.boolean(
                    'Autocomplete',
                    help="Will this order be automatically completed " +
                        "as soon as the raw materials are available?"),
                'origin': fields.char(
                    'Origin', 
                    size=64, 
                    help="Reference of the document that generated this " +
                        "production order request."),
                'routing_id': fields.many2one(
                    'mrp.routing', 
                    string='Routing', 
                    on_delete='set null', 
                    help="The list of operations (list of work centers) to " +
                        "produce the finished product. The routing is mainly " +
                        "used to compute work center costs during operations " +
                        "and to plan future loads on work centers based on " +
                        "production plannification.")}
    
    def create(self, cr, uid, vals, context=None):
        """ Create a production order record.
        
        Copies autocomplete field from the bom if it hasn't already been set.
        """
        
        if 'autocomplete' not in vals:
            bom_id = vals.get('bom_id', None)
            if bom_id:
                bom_obj = self.pool.get('mrp.bom')
                bom = bom_obj.browse(cr, uid, bom_id, context)
                vals['autocomplete'] = bom.autocomplete
        return super(mrp_production, self).create(cr, uid, vals, context)
    
    def write(self, cr, uid, ids, vals, context=None):
        """ Modify a production order record.
        
        Copies autocomplete field from the bom if it hasn't already been set.
        """
        
        if 'autocomplete' not in vals:
            bom_id = vals.get('bom_id', None)
            if bom_id:
                bom_obj = self.pool.get('mrp.bom')
                bom = bom_obj.browse(cr, uid, bom_id, context)
                vals['autocomplete'] = bom.autocomplete
        return super(mrp_production, self).write(cr, uid, ids, vals, context)
    
    def action_production_end(self, cr, uid, ids):
        for production in self.browse(cr, uid, ids):
            if production.autocomplete:
                self.action_produce(cr, 
                                    uid, 
                                    production.id, 
                                    production.product_qty, 
                                    production_mode='consume_produce')
        return super(mrp_production, self).action_production_end(cr, uid, ids)

    def _prepare_production_from_bom(self, cr, uid, bom, context=None):
        """ Prepare production order fields to update based on a BOM.
        
        bom - a browse object to copy details from
        returns a dictionary of field names and their new values.
        """
        updates = super(mrp_production, self)._prepare_production_from_bom(
            cr,
            uid,
            bom,
            context)
        updates['autocomplete'] = bom.autocomplete
        return updates

mrp_production()
