import time

from osv import fields, osv

AVAILABLE_STATES = [('unchanged', 'Unchanged'),
                    ('draft', 'New'),
                    ('open', 'In Progress'),
                    ('cancel', 'Cancelled'),
                    ('done', 'Closed'),
                    ('pending', 'Pending')]

class crm_add_history(osv.osv_memory):
    """Adds a new history entry to the case."""
    _name = 'crm.add.history'
    _description = "Add History"

    _columns = {
        'body': fields.text('Text', required=True),
        'entry_type_id': fields.many2one('crm.entry.type', 'Entry Type'),
        'state': fields.selection(AVAILABLE_STATES, 
                                  string='Set New State To',
                                  required=True),
    }

    _defaults = {
        'state': 'unchanged'
    }

    def action_add(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        model = 'crm.helpdesk'
        case_pool = self.pool.get(model)
        mail_message = self.pool.get('mail.message')

        for obj in self.browse(cr, uid, ids, context=context):
            case_list = case_pool.browse(cr, 
                                         uid, 
                                         context['active_ids'],
                                         context=context)
            case = case_list[0]
            data = {
                'subject': 'Historize',
                'user_id': uid,
                'model' : model,
                'res_id': case.id,
                'date': time.strftime('%Y-%m-%d %H:%M:%S'),
                'body_text': obj.body,
                'partner_id': case.partner_id.id,
                'entry_type_id': obj.entry_type_id.id,
                'state' : 'received',
            }
            mail_message.create(cr, uid, data, context=context)
            if obj.state == 'unchanged':
                pass
            elif obj.state == 'done':
                case_pool.case_close(cr, uid, [case.id])
            elif obj.state == 'draft':
                case_pool.case_reset(cr, uid, [case.id])
            elif obj.state in ['cancel', 'open', 'pending']:
                act = 'case_' + obj.state
                getattr(case_pool, act)(cr, uid, [case.id])

        return {'type': 'ir.actions.act_window_close'}

crm_add_history()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
