from osv import osv, fields
from datetime import datetime, timedelta

class make_procurement(osv.osv_memory):
    _inherit = 'make.procurement'
    
    _columns = {'date_planned': fields.datetime('Planned Date', 
                                                required=True),
                'is_old_product': fields.boolean('Is Old Product')}
    
    def default_get(self, cr, uid, fields, context=None):
        result = super(make_procurement, self).default_get(cr, 
                                                           uid, 
                                                           fields, 
                                                           context)
        product_id = result.get('product_id', None)
        if not product_id:
            return result
        product = self.pool.get('product.product').browse(cr, 
                                                          uid, 
                                                          product_id, 
                                                          context)
        if 'date_planned' in fields:
            date_planned = datetime.now() + timedelta(days=product.seller_delay)
            result['date_planned'] = date_planned.strftime('%Y-%m-%d %H:%M:%S')
        if 'is_old_product' in fields:
            result['is_old_product'] = product.state in ('end','obsolete')
        if 'warehouse_id' in fields:
            warehouse_ids = self.pool.get('stock.warehouse').search(cr, uid, [])
            if warehouse_ids and (len(warehouse_ids) == 1):
                result['warehouse_id'] = warehouse_ids[0]
        
        return result

    def make_procurement(self, cr, uid, ids, context=None):
        super(make_procurement, self).make_procurement(cr, 
                                                       uid, 
                                                       ids, 
                                                       context)
        return {'type': 'ir.actions.act_window_close'}
        
make_procurement()

