from osv import fields
from osv import osv

class purchase_order(osv.osv):
    '''
    Add the attention to field to a purchase order to indicate who needs the order
    Make sure that the order reference field is configured with the proper settings 
    (which purchase_manual breaks)
    Add the ability to include the PST footer information
    '''
    _inherit = 'purchase.order'

    _order = "shipped, minimum_planned_date, name desc"

    def _notes_shortened (self, cr, uid, ids, field_name, arg, context):
    # --------------------------------------------------
        ORDER_NOTES_TREE_VIEW_MAX_LENGTH = 30
        result = {}
        for order in self.browse(cr, uid, ids, context=context):
            notes = order.notes
            if not notes:
                result[order.id] = notes
            else:
                shortened = notes.partition('\n')[0]
                if ( len(shortened) < ORDER_NOTES_TREE_VIEW_MAX_LENGTH ): 
                    result[order.id] = shortened 
                else:
                    result[order.id] = shortened[:ORDER_NOTES_TREE_VIEW_MAX_LENGTH] + " ..."
        return result

    _columns = {
        'name': fields.char('Order Reference', size=64, required=True, select=True),
        'attention_to': fields.char(
            'Attention To', 
            help='Used to identify the individual who should receive the package',
            size=64),
        'notes_shortened': fields.function(
              _notes_shortened,
              type = 'char',
              method = True,
              string = 'Note'),
        'pst_exemption_purchase': fields.boolean(
                              'PST Exempt',
                              help='Append "Exempt from PST" note to PO'
                            ),
        'payment_term':fields.many2one(
              'account.payment.term',
              'Payment Method'),
    }
    
    _defaults = {'invoice_method': lambda *a: 'picking'}

    def onchange_supplier_po_profiles_partner_id(self, cr, uid, ids, partner_id, notes):
      result = self.onchange_partner_id(cr,uid,ids,partner_id)
      partner = self.pool.get('res.partner').browse(cr,uid,partner_id)

      notes_append = partner.notes_purchase

      result['value']['pst_exemption_purchase'] = partner.pst_exemption_purchase

      if not notes_append: return result

      if notes:
        result['value']['notes'] = notes+"\n"+notes_append
      else:
        result['value']['notes'] = notes_append

      return result


purchase_order()

class purchase_order_line(osv.osv):
    '''
    Make the "revision" of the current product being linked available in the PO screen

    Add the PO status
    Add the default product code
    Add the supplier's name
    Add the supplier's partnumber

    '''

    def _supplier_part_number(self, cr, uid, ids, name, arg, context={}):
        res = {}

        supply_obj = self.pool.get('product.supplierinfo') 
        for po_line in self.browse(cr, uid, ids, context):

            supply_ids = supply_obj.search(cr,uid,[
                            ('product_id','=',po_line.product_id.id),
                            ('name','=',po_line.partner_id.id)
                            ])
            if len(supply_ids):
                for supply_rec in supply_obj.browse(cr,uid,supply_ids,context):
                    res[po_line.id] = supply_rec.product_code
            else:
                res[po_line.id] = ""

        return res

    def _supplier_part_number_search(self, cr, uid, ids, name, args, context={}):
        '''
        Ugh. This is a bit ugly but it allows searching of PO numbers by currently active
        Supplier Partnumber. 
        '''

        # How are we filtering?
        op = args[0][1]

        # We only know one way of search
        if op != "ilike":
            logger = netsvc.Logger()
            logger.notifyChannel(
                'zerp_customizations', 
                netsvc.LOG_WARNING, 
                "Cannot handle operation '%s' on purchase.order.supplier_part_number field"
                                          % (op))
            return [('id','=','0')]

        # purchase.order.line 
        # where 
        # supplier_id + product_id = supplier_code ilike name
        # Because of how complicated it is, we cheat and directly use SQL
        val = args[0][2]
        cr.execute(
            """
                select distinct
                    pol.id
                from
                    product_supplierinfo as ps
                    inner join purchase_order_line as pol
                    on ( ps.product_id = pol.product_id )
                    inner join purchase_order as po
                    on ( ps.name = po.partner_id )

                where ps.product_code ilike %s
            """,
            [ "%"+val+"%" ]
        )

        search_ids = cr.fetchall()


        return [('id','in',search_ids)]

    _inherit = 'purchase.order.line'
    _columns = {

        'order_state': fields.related(
                          'order_id',
                          'state',
                          type='selection',
                          string='Order State',
                          selection=[
                            ('draft', 'Request for Quotation'),
                            ('wait', 'Waiting'),
                            ('confirmed', 'Confirmed'),
                            ('approved', 'Approved'),
                            ('except_picking', 'Shipping Exception'),
                            ('except_invoice', 'Invoice Exception'),
                            ('done', 'Done'),
                            ('cancel', 'Cancelled')]),
        'revision': fields.related(
                          'product_id',
                          'revision',
                          type='string',
                          relation='product.product',
                          string='Revision'
                        ),

        'product_default_code': fields.related(
                          'product_id',
                          'default_code',
                          type='char',
                          string='Product Default Code'
                        ),

        'supplier_part_number': fields.function(
                          _supplier_part_number, 
                          method=True, 
                          type='char', 
                          readonly=True,
                          fnct_search=_supplier_part_number_search, 
                          string='Supplier Code'
                        ),

        'partner_name': fields.related(
                          'order_id',
                          'partner_id',
                          'name',
                          type='char',
                          string='Supplier Name'
                        ),

    }
    
    def onchange_product_qty(self, *args, **kwargs):
        res =  self.product_id_change(*args, **kwargs)
        if 'product_qty' in res['value']:
            del res['value']['product_qty']
        if 'name' in res['value']:
            del res['value']['name']
        return res            
    
    def onchange_product_uom(self, 
                             cr, 
                             uid, 
                             ids, 
                             pricelist_id, 
                             product_id, 
                             *args, 
                             **kwargs):
        """ Because we allowed null product_id again, we have to skip the
        regular validation when product_id is null."""
        if not product_id:
            return {}
         
        res =  super(purchase_order_line, self).onchange_product_uom(
            cr,
            uid,
            ids,
            pricelist_id,
            product_id,
            *args, 
            **kwargs)
        
        if 'product_qty' in res['value']:
            del res['value']['product_qty']
        if 'name' in res['value']:
            del res['value']['name']
        return res            

purchase_order_line()
