import time
from mx.DateTime import now

from report import report_sxw
import pooler

class account_invoice(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(account_invoice, self).__init__(cr, uid, name, context=context)
        self.localcontext.update({
            'time': time,
            'po_number': self.po_number,
            'incoterms': self.incoterms,
            'payment_term_timeline': self.payment_term_timeline,
            'combined_lines': self.combined_lines,
        })

    def po_number(self,inv):
        '''
        Returns a comma delimited list of POs that this
        invoice has been associated with
        '''

        if not inv.order_ids:
            return ""

        # First, figure out the unique PO numbers
        po_numbers = set()
        for order in inv.order_ids:
            if order.client_order_ref:
                po_numbers.add(order.client_order_ref)

        # Then, build a string with the unique PO numbers
        po_number_str = ", ".join(po_numbers)
        return po_number_str


    def incoterms (self,inv):
        '''
        Returns a comma delimited list of POs that this
        invoice has been associated with
        '''

        if not inv.order_ids:
            return ""

        # First, figure out the unique Incoterms
        incoterm_codes = set()
        for order in inv.order_ids:
            if order.incoterm:
                incoterm_codes.add(order.incoterm.code)

        # Then, build a string with the unique Incoterms
        incoterm_str = ", ".join(incoterm_codes)
        return incoterm_str

    def payment_term_timeline(self,inv):
        '''
        Return an english language description of what
        must be performed for the payment term
        '''

        payterm = inv.payment_term

        # inv.date_invoice is actually of type report_sxw._date_format
        # when we do a test inv.date_invoice will not be False even though the value
        # may not hold a date. We force the string into a string which will guarantee that
        # we can get something consistant to test against
        if not inv.date_invoice or inv.date_invoice + "" == "False":
            date_invoice = now().strftime('%Y-%m-%d')+""
        else:
            date_invoice = inv.date_invoice

        payterm_schedule = self.pool.get('account.payment.term').compute(
                                            self.cr,
                                            self.uid,
                                            payterm.id,
                                            inv.amount_total,
                                            date_invoice,
                                            self.localcontext
                                            )
        payterm_res = []
        for pt in payterm_schedule:
            payterm_res.append({
                                      'amount': pt[1],
                                      'date': pt[0]
                                  })

        return payterm_res
    
    def combined_lines(self, invoice):
        invoice_obj = pooler.get_pool(self.cr.dbname).get('account.invoice')
        return invoice_obj._combined_lines(invoice)

report_sxw.report_sxw(
    'report.account.invoice.zaber',
    'account.invoice',
    'addons/zaber_report_account/report/account_print_invoice.rml',
    parser=account_invoice
)
