from osv import fields, osv

class move_duplicate(osv.osv_memory):
    _name = "zerp.account.move.duplicate"
    _description = "Duplicate Move"

    _columns = {'move_id': fields.many2one('account.move',
                                           'Account Move',
                                           required=True),
                'move_period_id': fields.many2one(
                    'account.period',
                    'Period',
                    required=True,
                    help='New Fiscal Period for this move'),
                'move_date': fields.date(
                    'Date',
                    required=True,
                    help='New effective date for the move and entries')}
    _defaults = {'move_date': fields.date.context_today}

    def default_get(self, cr, uid, fields, context=None):
        if context is None: context = {}
        res = super(move_duplicate, self).default_get(cr, uid, fields, context=context)
        move_ids = context.get('active_ids', [])
        if not move_ids or (not context.get('active_model') == 'account.move') \
            or len(move_ids) != 1:
            # Partial Picking Processing may only be done for one picking at a time
            return res
        move_id, = move_ids
        if 'move_id' in fields:
            res.update(move_id=move_id)
        return res

    def duplicate_move(self, cr, uid, ids, context=None):
        copy_am_id = None
        src_am_id = None
        am_obj = self.pool.get('account.move')
        aml_obj = self.pool.get('account.move.line')
        for params in self.browse(cr, uid, ids, context=context):
                
            # src_am_id = source account.move record id
            src_am_id = params.move_id.id
    
            # copy_am_id = copied account.move record id
            copy_am_id = am_obj.copy(cr,
                                     uid,
                                     src_am_id,
                                     {'period_id': params.move_period_id.id,
                                      'date': params.move_date},
                                     context)
            copy_am_rec = am_obj.browse(cr,uid,copy_am_id,context)
    
            # Set the date across all the account move lines as well
            # aml = account.move.line
            # Then recalculate all the credit/debit values if amount_currencies 
            # are found.
            for aml_rec in copy_am_rec.line_id:
                aml_update = {
                    'period_id': params.move_period_id.id,
                    'date':      params.move_date,
                }
    
                if aml_rec.amount_currency:
                    res = aml_obj.onchange_currency(cr,
                                                    uid,
                                                    [aml_rec.id],
                                                    aml_rec.account_id.id,
                                                    aml_rec.amount_currency,
                                                    aml_rec.currency_id.id,
                                                    aml_rec.date,
                                                    context=context)
                    value = res.get('value', None)
                    if value:
                        aml_update.update(value)
    
                aml_obj.write(cr, uid, [aml_rec.id], aml_update, context)

        return {
            'domain': "[]",
            'name': 'Duplicate of Account Move '+str(src_am_id),
            'view_type': 'form',
            'view_mode': 'form',
            'res_id': copy_am_id,
            'res_model': 'account.move',
            'context': '{}',
            'type': 'ir.actions.act_window'
        }
move_duplicate()
