from osv import fields, osv

class account_move_line(osv.osv):
    _inherit = "account.move.line"

    def compute_from_currency_amount(self, cr, uid, account_id, currency_id, effective_date, amount_currency):
        ''' Calculate what the credit/debit should be for an account move
            on a particular day based upon the amount from the target currency
        '''

        if not currency_id:
            return

        # If the account id has not been specified, let's see if we can
        # find out anyways
        base_currency_id = None

        if not account_id:
            cr.execute('''
              SELECT   company_id
              FROM     account_account
              GROUP BY company_id
            ''')
            company_ids = [row[0] for row in cr.fetchall()]
            if len(company_ids) > 1:
                return
            base_currency_id = self.pool.get('res.company').browse(cr,uid,company_ids[0]).currency_id.id
        else:
            account_obj = self.pool.get('account.account')
            account_rec = account_obj.browse(cr,uid,[account_id])[0]
            base_currency_id = account_rec.company_id.currency_id.id

        res_currency_obj = self.pool.get('res.currency')

        calc_value = res_currency_obj.compute(cr, uid,
                                                currency_id,
                                                base_currency_id, 
                                                amount_currency, 
                                                context={ 'date': effective_date } )

        if calc_value < 0:
            return {'credit': -calc_value,'debit':0}
        else:
            return {'credit': 0,'debit':calc_value}

    def onchange_amount_currency( self, cr, uid, ids, account_id, currency_id, effective_date, amount_currency ):
        ''' Return the data to autopopulate the currency amount with the appropriate 
            amount of base currency 
        '''
        res = self.compute_from_currency_amount(cr, uid, account_id, currency_id, effective_date, amount_currency)
        if res:
            return {'value':res}
        else:
            return {}



account_move_line()
