# -*- encoding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2011 Zaber Inc. All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


import db_external
from tools import config
import psycopg2
from psycopg2.psycopg1 import cursor as psycopg1cursor
from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT, ISOLATION_LEVEL_READ_COMMITTED, ISOLATION_LEVEL_SERIALIZABLE
import psycopg2.extensions
psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)


import pooler

RECORD_LIFESPAN = '1 month'
DB_KEY = 'fedex'
ZERP_VIEW_DATA = {
  'fedex_recipients': {
        'fields': {
            'pick_id': "sp.id",
            'pick_name': "sp.name",
            'name': "rpa.name",
            'company_name': "rp.name",
            'phone': "rpa.phone",
            'fax': "rpa.fax",
            'street': "rpa.street",
            'street2': "rpa.street2",
            'city': "rpa.city",
            'zip': "rpa.zip",
            'email': "rpa.email",
            'importer_tax_id': "rp.importer_tax_id",
            'state_code': "rcs.code",
            'country_code': "rc.code",
          },

          'from': '''
                      stock_picking as sp
            LEFT JOIN res_partner_address AS rpa ON sp.address_id = rpa.id
            LEFT JOIN res_partner AS rp ON rp.id = rpa.partner_id
            LEFT JOIN res_country AS rc ON rpa.country_id = rc.id
            LEFT JOIN res_country_state AS rcs on rpa.state_id = rcs.id
          ''',

          'where': '',

          'order': 'rpa.id',
  },

  'fedex_commodities': {
            'fields': {
                'move_id': "sm.id",
                'quantity': "sm.product_qty",
                'pick_id': "sp.id",
                'pick_name': "sp.name",
                'default_code': "pp.default_code",
                'description': "coalesce(pc.ship_description,pt.name)",
                'weight': "pt.weight",
                'harmonized_code': "pc.harmonized_Code",
                'country_of_origin': "rc.code",
                'unit_value': "pt.list_price",
                'total_customs_value': "(pt.list_price * sm.product_qty)",
                'product_uos': "product_uos",
                'product_uom': "pu.name",
                'delivery_carrier_name': "dc.name",
              },

              'from': '''
                          stock_move AS sm
                LEFT JOIN stock_picking AS sp ON sm.picking_id = sp.id
                LEFT JOIN product_product AS pp ON sm.product_id = pp.id
                LEFT JOIN product_template AS pt ON pp.product_tmpl_id = pt.id
                LEFT JOIN product_uom AS pu ON sm.product_uos = pu.id
                LEFT JOIN product_category as pc ON pt.categ_id = pc.id
                LEFT JOIN res_country as rc ON pc.origin_country_id = rc.id
                LEFT JOIN delivery_carrier AS dc on sp.carrier_id = dc.id
              ''',

              'where': '',

              'order': 'sm.id',
  },
}
FEDEX_TABLE_DATA = {
  'fedex_recipients': {
      'pk': 'pick_id',
      'fields': {
        'pick_id': 'integer primary key',
        'pick_name': 'varchar(255)',
        'name': 'varchar(255)',
        'company_name': 'varchar(255)',
        'phone': 'varchar(255)',
        'fax': 'varchar(255)',
        'street': 'varchar(255)',
        'street2': 'varchar(255)',
        'city': 'varchar(255)',
        'zip': 'varchar(255)',
        'email': 'varchar(255)',
        'importer_tax_id': 'varchar(255)',
        'state_code': 'varchar(2)',
        'country_code': 'varchar(2)'
      },
  },

  'fedex_commodities': {
      'pk': 'move_id',
      'fields': {
        'move_id': 'integer primary key',
        'quantity': 'float',
        'pick_id': 'integer',
        'pick_name': 'varchar(255)',
        'default_code': 'varchar(255)',
        'description': 'varchar(255)',
        'weight': 'float',
        'harmonized_code': 'varchar(100)',
        'country_of_origin': 'varchar(100)',
        'unit_value': 'float',
        'total_customs_value': 'float',
        'product_uos': 'float',
        'product_uom': 'varchar(255)',
        'delivery_carrier_name': 'varchar(255)'
      },
  },

}

def _zerp_pull_data(cr,uid,view_name,where,context):
    ''' Extract all the records associated with a view.
        This can potentially be a large query, however, since
        we only pull records that are active and strongly filtered
        we only expect a small number of results.
    '''
    view_data = ZERP_VIEW_DATA[view_name]
    view_cols = sorted(view_data['fields'].keys())
    view_cols_str = ",".join(view_cols)
    record_sql = 'SELECT %s FROM %s' % (view_cols_str,view_name)
    if where:
        record_sql += " WHERE " + where

    try:
        cr.execute(record_sql)
        return cr.fetchall()

    # If we get a programming error, it's probably because
    # the view doesn't exist yet. So, let's just create it
    # If the select fails a second time, the exception can
    # propagate up
    except psycopg2.ProgrammingError as e:
        cr.rollback()
        where = view_data['where']
        if where:
            where = "WHERE " + where
        view_create_columns = [defn + " AS " +name for name,defn in view_data['fields'].iteritems()]
        view_create_sql  = """
            CREATE OR REPLACE VIEW %s AS 
              SELECT %s
              FROM %s
              %s
              ORDER BY %s
           """ % ( view_name,
                   ",\n".join(view_create_columns),
                   view_data['from'],
                   where or '',
                   view_data['order'] )
        cr.execute(view_create_sql)
        cr.execute(record_sql)
        return cr.fetchall()

def _fedex_db():
    fedex_db = psycopg2.connect(
            database=config.get('external_'+DB_KEY+'_database'),
            host=config.get('external_'+DB_KEY+'_host'),
            port=config.get('external_'+DB_KEY+'_port'),
            user=config.get('external_'+DB_KEY+'_user'),
            password=config.get('external_'+DB_KEY+'_password'),
          )
    return fedex_db

def _fedex_push_data(cr,uid,view_name,data_rows,context):
    view_data = ZERP_VIEW_DATA[view_name]
    view_cols = sorted(view_data['fields'].keys())
    view_cols_str = ",".join(view_cols)

    fedex_view_data = FEDEX_TABLE_DATA[view_name]
    fedex_db = _fedex_db()
    fedex_cur = fedex_db.cursor(cursor_factory=psycopg1cursor)

    try:
        fedex_cur.execute(
          "DELETE FROM %s WHERE write_timestamp < NOW()- INTERVAL '%s'" % (view_name,RECORD_LIFESPAN),
          [RECORD_LIFESPAN]
        )
    except psycopg2.ProgrammingError as e:
        fedex_db = _fedex_db()
        fedex_cur = fedex_db.cursor(cursor_factory=psycopg1cursor)
        table_create_columns = [name+" "+defn for name,defn in fedex_view_data['fields'].iteritems()]
        table_create_columns.append("write_timestamp timestamp")
        table_create_sql = """CREATE TABLE %s (%s)""" % (
                              view_name,
                              ",\n".join(table_create_columns)
                            )
        fedex_cur.execute(table_create_sql)

    fedex_view_pk = fedex_view_data['pk']
    for hit_row in data_rows:
        pk_value = dict(zip(view_cols,hit_row))[fedex_view_pk]
        update_set = [
            col_name + "=%s"
              for col_name in view_cols
        ]
        update_set.append("write_timestamp=NOW()")

        update_sql = "UPDATE %s SET %s WHERE %s=%s" % (
                        view_name,
                        ",".join(update_set),
                        fedex_view_pk,
                        pk_value
                      )
        fedex_cur.execute(update_sql,hit_row)

        # Only insert if we find no records with a matching PK
        # We do this because UPSERT isn't available in postgresql
        insert_sql = """INSERT INTO %s
                          ( %s, write_timestamp )
                            SELECT %s, NOW()
                              WHERE 1 NOT IN ( SELECT 1 FROM %s WHERE %s=%%s )
                     """ % (
                        view_name,
                        ",".join(view_cols),
                        ",".join(["%s"] * len(view_cols)),
                        view_name,
                        fedex_view_pk
                     )

        row_data = list(hit_row)
        row_data.append(pk_value)
        fedex_cur.execute(insert_sql,row_data)

    fedex_db.commit()

def data_push (cr,uid,data,context):
    if not config.get('external_'+DB_KEY+'_database',False):
      return

    where = 'pick_id = %s' % (data['id'])
    fedex_recipients = _zerp_pull_data(cr,uid,'fedex_recipients',where,context)
    _fedex_push_data(cr,uid,'fedex_recipients',fedex_recipients,context)

    where = 'pick_id = %s' % (data['id'])
    fedex_commodities = _zerp_pull_data(cr,uid,'fedex_commodities',where,context)
    _fedex_push_data(cr,uid,'fedex_commodities',fedex_commodities,context)

