def merge_provisions(c):

    def _merge_preview(orm,cr,uid,master_id,merge_ids):
        """ Build a record that describes what information gets updated
            if we try and merge the merge_ids into the master_id
        """

        moving_references = {}

        # The following will be used to discover/note field differences 
        # betwixt master record and merge records
        master_rec = orm.browse(cr,uid,master_id)
        columns = master_rec._table._columns
        updates = {}
        differences = {}
        friendly_names = {} # id -> name

        # Okay, for each merge id we must do a dry run of a merge...
        for merge_id in merge_ids:

            # Sanity! Merging unto itself?
            if merge_id == master_id: continue

            # This chunk will determine what mail messages
            # will require moving over
            moves = {}

            merge_rec = orm.browse(cr,uid,merge_id)
            friendly_names[merge_id] = "%d: %s"%(merge_id,merge_rec.name)
            
            if merge_rec.message_ids:
                mails = {}
                for msg in merge_rec.message_ids:
                    mails[msg.id] = {'mail: subject': msg.subject}
                    moves['mail_message'] = mails
            
            # This chunk will determine what attachments need
            # to be moved over
            ir_att_obj = orm.pool.get("ir.attachment")
            att_ids = ir_att_obj.search(cr, uid, [('res_model','=',orm._name),('res_id','=',merge_id)])
            if att_ids:
                atts = {}
                for att_rec in ir_att_obj.browse(cr, uid, att_ids):
                    atts[att_rec.id] = {'attachment: name': att_rec.name}
                moves['ir_attachment'] = atts
            
            if moves:
                moving_references[merge_id] = moves

            # This chunk will figure out what differences there are between
            # the master record and merge records
            for col_name,col_type in columns.iteritems():
                if (not col_type._classic_read
                    or col_name == "message_ids"
                    or col_name == "write_date"
                    or not merge_rec[col_name] 
                    or master_rec[col_name] == merge_rec[col_name]): 
                        continue
                if master_rec[col_name] or getattr(updates,col_name,False):
                    differences.setdefault(merge_id,{})[col_name] = merge_rec[col_name]
                else:
                    updates[col_name] = merge_rec[col_name]


        return {
            'moving_references': moving_references,
            'updates': updates,
            'differences': differences,
            'friendly_names': friendly_names,
        }

    def _merge_preview_str(orm,cr,uid,master_id,merge_ids):
        merge_str = ''
        merge_preview = orm.merge_preview(cr,uid,master_id,merge_ids)
        friendly_names = merge_preview['friendly_names']

        if merge_preview['updates']:
            merge_str += '--------------------------------------------------\n'
            merge_str += 'The following updates will be performed on the main record:\n'
            for field,value in merge_preview['updates'].iteritems():
                merge_str += "    %s = '%s'\n" % (field,value)
            merge_str += "\n"

        if merge_preview['differences']:
            merge_str += '--------------------------------------------------\n'
            merge_str += 'The following differences will be noted for posterity:\n'
            for merge_source,merge_differences in merge_preview['differences'].iteritems():
                merge_str += "\nMerge Source: %s\n" % (friendly_names[merge_source],)
                for field,value in merge_differences.iteritems():
                    merge_str += "    %s = '%s'\n" % (field,value)
            merge_str += "\n"


        if merge_preview['moving_references']:
            merge_str += '--------------------------------------------------\n'
            merge_str += 'The following data will be moved to the main record:\n'
            for merge_source,tables in merge_preview['moving_references'].iteritems():
                merge_str += "\nMerge Source: %s\n" % (friendly_names[merge_source],)
                for table, moves in tables.iteritems():
                    for id, move in moves.iteritems():
                        for field,value in move.iteritems():
                            merge_str += "    %s = '%s'\n" % (field,value)
            
        if not merge_str:
            merge_str = "No differences or merges required. We will just remove the merge records."

        return merge_str

    def _merge_records(orm,cr,uid,master_id,merge_ids):

        merge_ids = [id for id in merge_ids if id != master_id]
        root_uid = 1 # required to force through some reallocation of references
        merge_preview = orm.merge_preview(cr,uid,master_id,merge_ids)

        # Move any references pointing to the old records to 
        # the master record
        if merge_preview['moving_references']:
            field_name = "res_id"
            for merge_source,tables in merge_preview['moving_references'].iteritems():
                for table, moves in tables.iteritems():
                    update_ids = map(str,moves.keys())
                    in_list = ",".join(update_ids)
                    update_sql = """
                      UPDATE %s 
                      SET %s = %d 
                      WHERE id in(%s)
                    """ % ( table, field_name, master_id, in_list )
                    cr.execute(update_sql)

        # Apply any requisite updates to the master id
        updates = merge_preview['updates']
        if merge_preview['differences']:
            master_rec = orm.browse(cr,uid,master_id)
            notes = updates.get(orm._notes_field) or master_rec[orm._notes_field] or ''
            notes += '\n--------------------------------------------------\n'
            notes += 'The following differences were detected at the time of merge:\n'
            for merge_source,merge_differences in merge_preview['differences'].iteritems():
                notes += "\nMerge Source: %s\n" % (merge_source,)
                for field,value in merge_differences.iteritems():
                    notes += "    %s = '%s'\n" % (field,value)
            notes += "\n"
            updates[orm._notes_field] = notes

        if updates:
            orm.write(cr,uid,master_id,updates)

        # We mark the merged records as inactive (don't delete)
        orm.write(cr,uid,merge_ids,{'active':False})

    c.merge_preview = _merge_preview
    c.merge_preview_str = _merge_preview_str
    c.merge_records = _merge_records

    return c

