# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2012 Zaber Technologies Inc. (<http://zaber.com>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import pooler
from tools.misc import UpdateableStr, UpdateableDict
from xml.sax.saxutils import escape

parameter_form = UpdateableStr()
parameter_fields = UpdateableDict()


class wizard_applicant_merge(wizard.interface):

    def _init_form(self, cr, uid, data, context):
        id = data['id']
        pool = pooler.get_pool(cr.dbname)
        applicant = pool.get('hr.applicant').browse(cr,uid,id)

        parameter_form.string = '''<?xml version="1.0"?>
            <form string="Merge Applicant?" colspan="4">
                <label string="Using applicant '%s' as the base record." colspan="4" align="0.0" />
                <label string="Which other applicant should be merged into this record?" colspan="4" align="0.0" />
                <label string="" colspan="4" align="0.0" />
                <field name="merge_id" />
            </form>
        ''' % ( escape(applicant.name) )
        parameter_fields.dict = {
            'merge_id': {
                'string':'Merge with Applicant', 
                'type':'many2one', 
                'relation':'hr.applicant',
                'required': True,
                'domain': "[('id','!=',%d)]" % (id,)
            }
        }
        
        return {}


    def _merge_preview(self, cr, uid, data, context):
        """
        Input from looks like:

        {
          'form': {'merge_id': 1234}, 
          'ids': [123], 
          'report_type': 'pdf', 
          'model': u'hr.applicant', 
          'id': 123
        }

        """
        pool = pooler.get_pool(cr.dbname)

        master_id = data['id']
        merge_ids = [data['form']['merge_id']]
        applicant_obj = pool.get('hr.applicant')
        merge_preview_str = applicant_obj.merge_preview_str(cr,uid,master_id,merge_ids)


        parameter_form.string = '''<?xml version="1.0"?>
            <form string="Confirm Applicant Merge" colspan="4">
                <field name="merge_data" nolabel="1" colspan="4" align="0.0" width="650" height="400" readonly="1" />
            </form>
        '''
        parameter_fields.dict = {
            'merge_data': {
                'type':'text', 
            }
        }
        return {
            'merge_data': merge_preview_str,
            'merge_ids': merge_ids,
        }
        

    def _merge(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        master_id = data['id']
        merge_ids = data['form']['merge_ids']
        pool.get('hr.applicant').merge_records(cr,uid,master_id,merge_ids)
        return {}
    

    states = {

        'init': {
            'actions': [_init_form],
            'result': {
                'type': 'form',
                'arch':parameter_form,
                'fields':parameter_fields,
                'state':[('end','Cancel'),('merge_preview','Merge Preview')]
            },
        },


        'merge_preview': {
            'actions': [_merge_preview],
            'result': {
                'type': 'form',
                'arch':parameter_form,
                'fields':parameter_fields,
                'state':[('end','Cancel'),('merge','Merge')]
            }
        },


        'merge': {
            'actions': [_merge],
            'result': {'type':'state','state':'end'}
        },

    }

wizard_applicant_merge('zerp.applicant.merge')


