def merge_provisions(c):

    def _merge_preview(orm,cr,uid,master_id,merge_ids):
        """ Build a record that describes what information gets updated
            if we try and merge the merge_ids into the master_id
        """

        # Get a list of _just_ tables (so we can ignore views)
        cr.execute('''
            select c.relname FROM pg_catalog.pg_class c
            LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
            WHERE c.relkind IN ('r','') AND n.nspname NOT IN ('pg_catalog', 'pg_toast')
            AND pg_catalog.pg_table_is_visible(c.oid);
        ''')
        database_tables = set()
        for table_data in cr.fetchall():
            database_tables.add(table_data[0])

        moving_references = {}

        # ir.model.fields holds all the relationships between tables that
        # we're interested in. Mostly the many2one fields so we can figure
        # out what tables need an update to "repoint" records from the old
        # merge_id to the new master_id
        model_fields_obj = orm.pool.get('ir.model.fields')
        root_uid = 1 # required to force through some reallocation of references
        model_field_ids = model_fields_obj.search(cr,root_uid,[
          ('relation','=',orm._name),
          ('ttype','=','many2one')
        ])
        model_field_recs = model_fields_obj.browse(cr,uid,model_field_ids)

        # The following will be used to discover/note field differences 
        # betwixt master record and merge records
        master_rec = orm.browse(cr,uid,master_id)
        columns = master_rec._table._columns
        updates = {}
        differences = {}
        friendly_names = {} # id -> name

        # Okay, for each merge id we must do a dry run of a merge...
        for merge_id in merge_ids:

            # Sanity! Merging unto itself?
            if merge_id == master_id: continue

            # This chunk will determine what references to the merge
            # records will require moving over
            moves = {}
            for model_field in model_field_recs:
                model = model_field.model
                name  = model_field.name
                orm_obj = orm.pool.get(model)

                # Sanity, just in case some cruft is left in the field recs
                if not orm_obj: continue

                # If we search based upon a field that's a one2many but
                # it's actually a method, it will cause the search to return
                # /all/ results. Pretty nasty if we hit stock.move 
                # There's one exception, that is, if we encounter a related
                # field with a stored value. Then not only do we have the
                # ability to execute queries on the field but we must 
                # update the field to the merge target
                if getattr(orm_obj._columns[name],'_fnct',False) \
                   and not getattr(orm_obj._columns[name],'store',False):
                    continue

                # Check that the ORM isn't referencing a view under the hood
                table_name = model.replace('.','_')
                if table_name not in database_tables: continue

                orm_merge_ids = orm_obj.search(cr,root_uid,
                                  [(name,'=',merge_id)],
                                  context={'active_test':False}
                                )
                orm_merge_recs = orm_obj.browse(cr,root_uid,orm_merge_ids)
                try:
                    orm_merge_names = orm_obj.name_get(cr,uid,orm_merge_ids)

                # NotImplemented means that we've got a many2one field that's
                # hooked onto an osv.osv_memory object. It's used with wizards
                # so we won't really worry about it, instead provide ids
                except NotImplementedError: 
                    orm_merge_names = dict(zip(orm_merge_ids,orm_merge_ids))

                if orm_merge_names:
                    moves.setdefault(model,{})[name] = dict(orm_merge_names)

            merge_rec = orm.browse(cr,uid,merge_id)
            friendly_names[merge_id] = "%d: %s"%(merge_id,merge_rec.name)
            if moves:
                moving_references[merge_id] = moves

            # This chunk will figure out what differences there are betwixt
            # the master record and merge records
            for col_name,col_type in columns.iteritems():
                if (not col_type._classic_read
                    or not merge_rec[col_name] 
                    or master_rec[col_name] == merge_rec[col_name]): 
                        continue
                if master_rec[col_name] or getattr(updates,col_name,False):
                    differences.setdefault(merge_id,{})[col_name] = merge_rec[col_name]
                else:
                    updates[col_name] = merge_rec[col_name]


        return {
            'moving_references': moving_references,
            'updates': updates,
            'differences': differences,
            'friendly_names': friendly_names,
        }

    def _merge_preview_str(orm,cr,uid,master_id,merge_ids):
        merge_str = ''
        merge_preview = orm.merge_preview(cr,uid,master_id,merge_ids)
        friendly_names = merge_preview['friendly_names']

        if merge_preview['updates']:
            merge_str += '--------------------------------------------------\n'
            merge_str += 'The following updates will be performed on the main record:\n'
            for field,value in merge_preview['updates'].iteritems():
                merge_str += "    %s = '%s'\n" % (field,value)
            merge_str += "\n"

        if merge_preview['differences']:
            merge_str += '--------------------------------------------------\n'
            merge_str += 'The following differences will be noted for posterity:\n'
            for merge_source,merge_differences in merge_preview['differences'].iteritems():
                merge_str += "\nMerge Source: %s\n" % (friendly_names[merge_source],)
                for field,value in merge_differences.iteritems():
                    merge_str += "    %s = '%s'\n" % (field,value)
            merge_str += "\n"


        if merge_preview['moving_references']:
            merge_str += '--------------------------------------------------\n'
            merge_str += 'The following data will be moved to the main record:\n'
            for merge_source,moves in merge_preview['moving_references'].iteritems():
                merge_str += "\nMerge Source: %s\n" % (friendly_names[merge_source],)
                for model, model_fields in moves.iteritems():
                    for field, merge_recs in model_fields.iteritems():
                        merge_str += "\n    %s::%s\n" % (model,field,)
                        for rec_id,rec_name in merge_recs.iteritems():
                            merge_str += "       %d: %s\n" % (rec_id,rec_name,)

        if not merge_str:
            merge_str = "No differences or merges required. We will just remove the merge records."

        return merge_str

    def _merge_records(orm,cr,uid,master_id,merge_ids):

        merge_ids = [id for id in merge_ids if id != master_id]
        root_uid = 1 # required to force through some reallocation of references
        merge_preview = orm.merge_preview(cr,uid,master_id,merge_ids)

        # Move any references pointing to the old records to 
        # the master record
        if merge_preview['moving_references']:
            for merge_id,moves in merge_preview['moving_references'].iteritems():
                for model,moves_by_field in moves.iteritems():
                    orm_obj = orm.pool.get(model)
                    table_name = model.replace('.','_')
                    for field_name,move_rec in moves_by_field.iteritems():

                        # Using an update query allows us to skirt around some
                        # validation code.
                        update_ids = map(str,move_rec.keys())
                        in_list = ",".join(update_ids)
                        update_sql = """
                          UPDATE %s 
                          SET %s = %d 
                          WHERE id in(%s)
                        """ % ( table_name, field_name, master_id, in_list )
                        cr.execute(update_sql)

        # Apply any requisite updates to the master id
        updates = merge_preview['updates']
        if merge_preview['differences']:
            master_rec = orm.browse(cr,uid,master_id)
            notes = updates.get(orm._notes_field) or master_rec[orm._notes_field] or ''
            notes += '\n--------------------------------------------------\n'
            notes += 'The following differences were detected at the time of merge:\n'
            for merge_source,merge_differences in merge_preview['differences'].iteritems():
                notes += "\nMerge Source: %s\n" % (merge_source,)
                for field,value in merge_differences.iteritems():
                    notes += "    %s = '%s'\n" % (field,value)
            notes += "\n"
            updates[orm._notes_field] = notes

        if updates:
            orm.write(cr,uid,master_id,updates)

        # We mark the merged records as inactive (don't delete)
        orm.write(cr,uid,merge_ids,{'active':False})

    c.merge_preview = _merge_preview
    c.merge_preview_str = _merge_preview_str
    c.merge_records = _merge_records

    return c

