# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2012 Zaber Technologies Inc. (<http://zaber.com>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import pooler
from tools.misc import UpdateableStr, UpdateableDict
from xml.sax.saxutils import escape
import re

parameter_form = UpdateableStr()
parameter_fields = UpdateableDict()


class wizard_partner_address_merge(wizard.interface):

    def _init_form(self, cr, uid, data, context):
        id = data['id']
        pool = pooler.get_pool(cr.dbname)
        partner = pool.get('res.partner').browse(cr,uid,id)
        address_obj = pool.get('res.partner.address')
        address_ids = address_obj.search(cr,uid,[('partner_id','=',id)])

        address_fields = {
            'master_id': {
                'string':'Main Contact', 
                'type':'many2one', 
                'required': True,
                'relation':'res.partner.address',
                'domain': "[('partner_id','=',%d)]" % (id,)
            },
        }
        address_data = {}
        address_list_xml = ''
        for address in address_obj.browse(cr,uid,address_ids):
            field_name = 'address_%d'%(address.id)
            address_fields[field_name] = {
                'string': 'Main Address',
                'type':'many2one', 
                'required': True,
                'readonly': True,
                'relation':'res.partner.address',
            }
            address_fields["merge_"+field_name] = {
                'type':'boolean', 
            }
            address_data[field_name] = address.id
            address_data["merge_"+field_name] = False
            address_list_xml += """
              <group colspan="4" attrs="{'invisible':[('master_id','=',%d)]}">
                <field name="merge_%s" colspan="1" align="1.0" nolabel="1" width="20" />
                <field name="%s" colspan="3" nolabel="1" width="600" />
              </group>
            """ % (address.id,field_name,field_name,)

        parameter_form.string = '''<?xml version="1.0"?>
            <form string="Which Contacts would you like to merge?" col="4">
                <label string="Merging contacts for partner '%s'" colspan="4" align="0.0"/>
                <label string="" colspan="4" align="0.0" />
                <group col="4" string="Main address: Which address will take all the merged addresses?" colspan="4">
                  <field name="master_id" nolabel="1" colspan="4" align="0.0" />
                  <label string="" colspan="4" align="0.0" />
                </group>
                <group col="4" string="Merge addresses: Select addresses to be merged into the main address" colspan="4">
                  %s
                </group>
            </form>
        ''' % ( escape(partner.name), address_list_xml )

        parameter_fields.dict = address_fields
        return address_data


    def _merge_preview(self, cr, uid, data, context):
        """
        Input from looks like:

        {
          'form': {
            'master_id': 20181,
            'merge_address_20254': 0,
            'merge_address_20255': 1,
            'address_13371': 13371,
            'address_13374': 13374,
          },
          'ids': [7114],
          'report_type': 'pdf',
          'model': u'res.partner',
          'id': 7114
        }

        """

        # Figure out what records we want to merge with what. Pretty much we want to get
        # master_id = address.id of target address to merge into
        # merge_address_ids = [1,2,3,...] ids of addresses to merge into the master
        form = data['form']
        master_id = form['master_id']
        merge_address_ids = []
        for field,value in form.iteritems():
            matched = re.search('merge_address_(\d+)',field)
            if not matched: continue
            if not value: continue
            merge_address_ids.append(int(matched.group(1)))

        # With the information provided, let's see what's different by
        # performing a dry run
        pool = pooler.get_pool(cr.dbname)
        address_obj = pool.get('res.partner.address')
        merge_preview_str = address_obj.merge_preview_str(cr,uid,master_id,merge_address_ids)

        # Build a human readable version of the data we've just calculated
        parameter_form.string = '''<?xml version="1.0"?>
            <form string="Confirm Partner Merge" colspan="4">
                <field name="merge_data" nolabel="1" colspan="4" align="0.0" width="650" height="400" readonly="1" />
            </form>
        '''
        parameter_fields.dict = {
            'merge_data': { 'type':'text' }
        }

        return {
            'merge_data': merge_preview_str,
            'merge_ids': merge_address_ids,
        }
        

    def _merge(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)
        master_id = data['form']['master_id']
        merge_ids = data['form']['merge_ids']
        address_obj = pool.get('res.partner.address')
        address_obj.merge_records(cr,uid,master_id,merge_ids)
        return {}

    def _init(self, cr, uid, data, context):
        id = data['id']
        pool = pooler.get_pool(cr.dbname)
        partner = pool.get('res.partner').browse(cr,uid,id)
        address_obj = pool.get('res.partner.address')
        address_count = address_obj.search(cr,uid,[('partner_id','=',id)],count=True)
        if address_count <= 1:
            return 'error_too_few'
        return 'merge_select'

    def _error_too_few(self, cr, uid, data, context):
        parameter_form.string = '''<?xml version="1.0"?>
            <form string="Merge Contacts" col="4">
                <label  string="Partner requires more than 1 address for merging to be useful." 
                        colspan="4" 
                        align="0.0" />
            </form>
        '''
        return {}


    states = {

        'init': {
          'actions': [],
          'result': { 'type': 'choice', 'next_state': _init }
        },

        'error_too_few': {
          'actions': [_error_too_few],
          'result': {
              'type': 'form',
              'arch':parameter_form,
              'fields':parameter_fields,
              'state':[('end','OK')]
          },
        },

        'merge_select': {
            'actions': [_init_form],
            'result': {
                'type': 'form',
                'arch':parameter_form,
                'fields':parameter_fields,
                'state':[('end','Cancel'),('merge_preview','Merge Preview')]
            },
        },


        'merge_preview': {
            'actions': [_merge_preview],
            'result': {
                'type': 'form',
                'arch':parameter_form,
                'fields':parameter_fields,
                'state':[('end','Cancel'),('merge','Merge')]
            }
        },


        'merge': {
            'actions': [_merge],
            'result': {'type':'state','state':'end'}
        },

    }

wizard_partner_address_merge('zerp.partner.address.merge')


