from osv import fields,osv
from tools.sql import drop_view_if_exists

class report_invoice_category(osv.osv):
    _name = "report.invoice.category"
    _description = "Customer Invoices by Product Category"
    _auto = False
    _columns = {
        'name': fields.date('Month', readonly=True),
        'category_id': fields.many2one('product.category', 'Categories', readonly=True),
        'quantity': fields.float('# of Products', readonly=True),
        'price_total': fields.float('Total Price', readonly=True),
        'price_average': fields.float('Average Price', readonly=True),
        'count': fields.integer('# of Lines', readonly=True),
    }
    _order = 'name desc,price_total desc'
    def init(self, cr):
        drop_view_if_exists(cr, 'report_invoice_category')
        cr.execute("""
            create or replace view report_invoice_category as (
                select  min(line.id) as id,
                        to_char(inv.date_invoice, 'YYYY-MM-01') as name,
                        tmpl.categ_id as category_id,
                        sum(line.quantity*uos.factor) as quantity,
                        count(*),
                        sum(line.quantity*line.price_unit) as price_total,
                        (sum(line.quantity*line.price_unit)/sum(line.quantity*uos.factor))::decimal(16,2) as price_average
                from    account_invoice inv
                join    account_invoice_line line
                on      inv.id = line.invoice_id
                left 
                join            product_product prod 
                        join    product_template tmpl 
                        on      tmpl.id = prod.product_tmpl_id
                on      prod.id = line.product_id
                left 
                join    product_uom uos 
                on      uos.id = line.uos_id
                where   line.quantity != 0
                and     inv.state in ('open', 'paid')
                and     inv.type = 'out_invoice'
                group 
                by      tmpl.categ_id, 
                        to_char(inv.date_invoice, 'YYYY-MM-01')
                having  sum(line.quantity*uos.factor) != 0
            )
        """)
report_invoice_category()


