import os
import re

import netsvc
import report, tools
from operator import itemgetter
import pooler

NODE_SEQ = 1

def graph_instance_get(cr, uid, graph, bom_rec, parent_node=None):
    '''
    '''
    import pydot
    db, pool = pooler.get_db_and_pool(cr.dbname)
    prod_obj = pool.get('product.product')
    bom_obj = pool.get('mrp.bom')

    if not bom_rec.bom_lines:
        newbom = bom_obj._bom_find(cr, uid, bom_rec.product_id.id, bom_rec.product_uom.id, [])
        if not newbom:
            return
        bom_rec = bom_obj.browse(cr,uid,newbom)

    for bom_entry in bom_rec.bom_lines:
        global NODE_SEQ
        NODE_SEQ += 1

        label = ""
        if bom_entry.product_id.default_code and len(bom_entry.product_id.default_code)<6:
            label += bom_entry.product_id.default_code+" "
        label += bom_entry.name+" "
        if bom_entry.product_id.name and bom_entry.product_id.name != bom_entry.name:
            label += bom_entry.product_id.name
        args = {
                'label': re.escape(label[0:40]),
                'shape': 'box',
              }
        child_node = pydot.Node(NODE_SEQ,**args)
        graph.add_node(child_node)

        # Draw an edge between the parent and child component
        args = {}
        if bom_rec.type == "phantom":
            args['style'] = 'dotted'
        graph.add_edge(pydot.Edge( 
                    child_node.get_name(), 
                    parent_node.get_name(),
                    fontsize='8', 
                    **args
                ))

        # Descend into the BoM to get the component's components
        graph_instance_get(cr,uid,graph,bom_entry,child_node)



class report_graph_instance(object):
    def __init__(self, cr, uid, ids, data):
        logger = netsvc.Logger()
        try:
            import pydot
        except Exception,e:
            logger.notifyChannel('workflow', netsvc.LOG_WARNING,
                    'Import Error for pydot, you will not be able to render workflows\n'
                    'Consider Installing PyDot or dependencies: http://dkbza.org/pydot.html')
            raise e
        self.done = False

        try:
            if not ids:
                mrp_bom = None
            else:
                product_id = ids[0]
                cr.execute('select * from mrp_bom where product_id=%s limit 1', (product_id,))
                mrp_bom = cr.dictfetchone()
            if not mrp_bom:
                ps_string = '''%PS-Adobe-3.0
/inch {72 mul} def

/Times-Roman findfont 30 scalefont setfont
1.5 inch 10 inch moveto
(No BoM for this product) show'''
            else:
                db, pool = pooler.get_db_and_pool(cr.dbname)
                prod_rec = pool.get('product.product').browse(cr,uid,product_id)


                graph = pydot.Dot(fontsize='16', label="""\\\n\\nProduct BoM: [%s] %s""" % (prod_rec.default_code, prod_rec.name),
                                  size='10.0, 7.3', center='1', ratio='auto', rotate='90', rankdir='LR', overlap='scale',
                                  splines='true',
                                 )

                args = {
                        'label': prod_rec.default_code,
                        'shape': 'box',
                        'style': 'filled',
                        'color': 'lightblue',
                      }
                parent_node = pydot.Node('parent',**args)
                graph.add_node(parent_node)

                bom_obj = pool.get('mrp.bom')
                bom_id = bom_obj._bom_find(cr,uid,prod_rec.id,prod_rec.uom_id.id)
                bom_rec = bom_obj.browse(cr,uid,bom_id)

                graph_instance_get(cr, uid, graph,bom_rec,parent_node)
                ps_string = graph.create(prog='neato', format='ps')

        except Exception, e:
            import traceback, sys
            tb_s = reduce(lambda x, y: x+y, traceback.format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback))
            logger.notifyChannel('workflow', netsvc.LOG_ERROR, 'Exception in call: ' + tb_s)
            # string is in PS, like the success message would have been
#            ps_string = '''%PS-Adobe-3.0
#1.00000 0.99083 scale
#/Courier findfont 12 scalefont setfont
#0 0 translate
#/row 769 def
#85 {/col 18 def 6 {col row moveto  /col col 90 add def}
#repeat /row row 9 sub def} repeat
#showpage save restore'''
            ps_string = '''%PS-Adobe-3.0
/inch {72 mul} def

/Times-Roman findfont 30 scalefont setfont
1.5 inch 10 inch moveto
(No BoM for this product) show'''
        if os.name == "nt":
            prog = 'ps2pdf.bat'
        else:
            prog = 'ps2pdf'
        args = (prog, '-', '-')
        input, output = tools.exec_command_pipe(*args)
        input.write(ps_string)
        input.close()
        self.result = output.read()
        output.close()
        self.done = True

    def is_done(self):
        return self.done

    def get(self):
        if self.done:
            return self.result
        else:
            return None


class report_graph(report.interface.report_int):
    def __init__(self, name, table):
        report.interface.report_int.__init__(self, name)
        self.table = table

    def result(self):
        if self.obj.is_done():
            return (True, self.obj.get(), 'pdf')
        else:
            return (False, False, False)

    def create(self, cr, uid, ids, data, context={}):
        self.obj = report_graph_instance(cr, uid, ids, data)
        return (self.obj.get(), 'pdf')

report_graph('report.bom.tree.graph', 'ir.workflow')

