# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Zaber Technologies Inc. All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from lxml import etree
import time
import math
from report import report_sxw
from osv import osv
import pooler
import pprint
import re

class StockValuate(report_sxw.rml_parse):

    def __init__(self, cr, uid, name, context):
        super(StockValuate, self).__init__(cr, uid, name, context=context)
        self.localcontext.update({
            'time': time,
        })

        # Additional frames seem to cause problems: Everything after the 2nd page
        # ends up printed on the 2nd page. Very illegible :(
        self.rml_header = re.sub('<frame id="(?!first).*?>',"",self.rml_header)
        self.context = context

    def set_context(self, objects, data, ids, report_type = None):
        ''' To avoid having to use repeatIn to set the valuation lines, we want to 
            precalculate the lines. Unfortunately, unless we hook into the set_context,
            we don't actually know what partner is being fetched. So we do the work here
            to figure it out.

            Note: this function could use some cleanup. There's copy pasta from
                  stock/product.py for the stock/value calculations in here and some
                  of it has been made redundant.
        '''


        if objects:
            sl_rec = objects[0]
          
            sl_obj = pooler.get_pool(self.cr.dbname).get('stock.location')

            location_ids = sl_obj.search(self.cr, self.uid, [('location_id', 'child_of', [sl_rec.id] )])
            location_ids_str = ','.join(map(str, location_ids))

            date_str = ""
            date_args = ()
            if "form" in data and "stock_date" in data["form"]:
                date_str = " and date_expected <= %s "
                date_args = ( data["form"]["stock_date"], )

            # Stock going in
            states = ['done']
            states_str = ','.join(map(lambda s: "'%s'" % s, states))
            cr = self.cr
            cr.execute(
                'select sum(product_qty), product_id, product_uom '\
                'from stock_move '\
                'where location_id not in %s '\
                'and location_dest_id in %s '\
                'and state in %s '+ date_str + ' '\
                'group by product_id,product_uom',
                (tuple(location_ids), tuple(location_ids), tuple(states))+date_args)

            results = cr.fetchall()

            # Stock going out
            cr.execute(
                'select sum(product_qty), product_id, product_uom '\
                'from stock_move '\
                'where location_id in %s '\
                'and location_dest_id not in %s '\
                'and state in %s '+ date_str + ' '\
                'group by product_id,product_uom',
                (tuple(location_ids), tuple(location_ids), tuple(states))+date_args)
            results2 = cr.fetchall()

            # Get a list of all the product IDs that have been involved in this
            # time period
            ids = set()
            for amount, prod_id, prod_uom in results:
                ids.add(prod_id)
            for amount, prod_id, prod_uom in results2:
                ids.add(prod_id)

            # Need to lookup UOM information for all the products
            # that have done something during this time period
            uoms = map(lambda x: x[2], results) + map(lambda x: x[2], results2)
            uoms_o = {}
            uom_obj = self.pool.get('product.uom')

            uid = self.uid
            if uoms:
                uoms = uom_obj.browse(cr, uid, list(set(uoms)))
            for o in uoms:
                uoms_o[o.id] = o

            product_obj = self.pool.get('product.product')
            product2uom = {}
            product_data = {}
            for product in product_obj.browse(cr, uid, list(ids)):
                product2uom[product.id] = product.uom_id.id
                uoms_o[product.uom_id.id] = product.uom_id
                product_data[product.id] = {
                                        'id': product.id,
                                        'default_code': product.default_code,
                                        'name': product.name,
                                        'standard_price': product.standard_price,
                                        'cost': 0.0,
                                        'qty': 0.0,
                                    }

            # Tabulate!
            for amount, prod_id, prod_uom in results:
                amount = uom_obj._compute_qty_obj(cr, uid, uoms_o[prod_uom], amount,
                        uoms_o[product2uom[prod_id]])
                product_data[prod_id]['cost'] += amount * product_data[prod_id]['standard_price']
                product_data[prod_id]['qty']  += amount
            for amount, prod_id, prod_uom in results2:
                amount = uom_obj._compute_qty_obj(cr, uid, uoms_o[prod_uom], amount,
                        uoms_o[product2uom[prod_id]])
                product_data[prod_id]['cost'] -= amount * product_data[prod_id]['standard_price']
                product_data[prod_id]['qty']  -= amount

            # Create a record that we can iterate over (and along the way
            # calculate the sum value of inventory)
            total_cost    = 0.0
            product_lines = []
            for prod_id,prod_rec in product_data.iteritems():
                if prod_rec['qty'] == 0:
                    continue
                total_cost += prod_rec['cost']
                product_lines.append(prod_rec)

            product_lines = sorted(product_lines,key=lambda l: l['cost'], reverse=True)
            self.localcontext.update({
                'total_cost': total_cost,
                'product_lines': product_lines,
            })

        return super(StockValuate, self).set_context(objects, data, ids, report_type)

report_sxw.report_sxw(
        'report.zerp.report.stock', 
        'stock.location',
        'addons/zerp_report_stock/report/stock.rml', 
        parser=StockValuate)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

