from osv import fields,osv

class zerp_yearly_sales_per_partner(osv.osv):
    _name = "zerp.yearly.sales.per.partner"
    _rec_name = "year"
    _columns = {
        'year': fields.integer('Year'),
        'partner_id': fields.many2one('res.partner', 'Customer', readonly=True),
        'amount_total': fields.float('Total Amount'),
        'country_id': fields.many2one('res.country', 'Country'),
        'country_name': fields.related(
                            'country_id', 
                            'name', 
                            type='string', 
                            string='Country'),

    }
    _order = 'year'

    def reload(self,cr,uid,context=None):
        ''' Reload all the data in the table
        '''

        # Remove the dataset first. Makes things easier
        self.unlink(cr,uid,self.search(cr,uid,[]))

        # Build the dataset we want to be using
        cr.execute('''

          select
              EXTRACT(YEAR FROM ai.date_invoice) as year,
              ai.partner_id,
              sum( /* Add or subtract from the yearly total based 
                      upon entry being an invoice or a refund*/
                abs(coalesce(aml.debit,0)+coalesce(aml.credit,0)) *
                case ai.type when 'out_refund' then -1
                                               else 1
                                               end
              ) as amount_total,
              coalesce(
                (
                select  min(country_id)
                from    res_partner_address addr
                where   addr.type = 'invoice'
                and     addr.partner_id = ai.partner_id
                ),
                (
                select  min(country_id)
                from    res_partner_address addr
                where   addr.type = 'default'
                and     addr.partner_id = ai.partner_id
                ),
                (
                select  min(country_id)
                from    res_partner_address addr
                where   addr.partner_id = ai.partner_id
                )) as country_id
          from
            account_invoice as ai
              left join
            account_move_line as aml on ( ai.move_id = aml.move_id )
              inner join
            account_account as aa  on ( 
                aml.account_id = aa.id and aa.type = 'receivable' 
                )
              inner join
            ( /* Get a list of all the customers that have purchased from us previously */
              select 
                rp.id as partner_id,
                rp.name as partner_name,
                min(extract(year from ai.date_invoice)) as year
              from
                res_partner as rp,
                account_invoice as ai
              where
                    rp.sales_report_exclude = False
                and rp.id = ai.partner_id
                and ai.type in('out_invoice')
                and ai.state not in('cancel','draft')
              group by 
                rp.id, rp.name
            ) as pl
            on (
                  pl.partner_id = ai.partner_id
                )
          where
            ai.type in('out_invoice','out_refund') 
            and ai.state not in('cancel','draft')
            and ai.amount_total <> 0
          group by 
            EXTRACT(YEAR FROM ai.date_invoice), 
            ai.partner_id, pl.partner_name

        ''')
        results = cr.fetchall()


        for partner_year_sale in results:
            new_partner_yearly_record = {
                'year':         partner_year_sale[0],
                'partner_id':   partner_year_sale[1],
                'amount_total': partner_year_sale[2],
                'country_id':   partner_year_sale[3],
            }
            self.create(cr,uid,new_partner_yearly_record,context)

zerp_yearly_sales_per_partner()



class zerp_yearly_sales(osv.osv):
    _name = "zerp.yearly.sales"
    _columns = {
        'year': fields.integer('Year'),
        'new_total': fields.float('New Sales'),
        'total': fields.float('Total Sales'),
        'new_partners': fields.integer('New Partners'),
        'total_partners': fields.integer('Total Partners'),
    }
    _order = 'year'

    def reload(self,cr,uid,context=None):
        ''' Reload all the data in the table
        '''

        # Remove the dataset first. Makes things easier
        self.unlink(cr,uid,self.search(cr,uid,[]))

        # Build the dataset we want to be using
        cr.execute('''

            select
              ncc.year,
              spy.new_total,
              spy.total,
              ncc.new_customer_count,
              tcc.total_customer_count
            from
              ( /* Figure out how many different customers made purchases on a particular year */
                  select year, count(*) as total_customer_count
                  from (
                      select distinct 
                        EXTRACT(YEAR FROM ai.date_invoice) as year, partner_id
                      from
                        account_invoice as ai,
                        res_partner as rp
                      where
                        ai.type in('out_invoice') 
                        and rp.sales_report_exclude = False
                        and rp.id = ai.partner_id
                        and ai.state not in('cancel','draft')
                        and ai.amount_total <> 0
                        ) as pl
                  group by year
              ) as tcc

              left join

              ( /* Count how many new customers we got on a particular year */
                select 
                  year, count(*) as new_customer_count
                from
                  (
                    select 
                      rp.id,
                      min(extract(year from ai.date_invoice)) as year
                    from
                      res_partner as rp,
                      account_invoice as ai
                    where
                          rp.sales_report_exclude = False
                      and rp.id = ai.partner_id
                      and ai.type in('out_invoice')
                      and ai.state not in('cancel','draft')
                      and ai.amount_total <> 0
                    group by 
                      rp.id
                  ) as pl
                group by 
                  year
              ) as ncc

              on ( tcc.year = ncc.year )

              left join

              ( /* Fetch the total purchases from new,repeat,all customers for the year */
                select
                    EXTRACT(YEAR FROM ai.date_invoice) as year,

                    sum(
                      case EXTRACT(YEAR FROM ai.date_invoice) 
                        when pl.year then
                            abs(coalesce(aml.debit,0)+coalesce(aml.credit,0)) *
                            case ai.type when 'out_refund' then -1
                                                           else 1
                                                           end
                        else 0
                        end
                    ) as new_total,

                    sum(
                      abs(coalesce(aml.debit,0)+coalesce(aml.credit,0)) *
                      case ai.type when 'out_refund' then -1
                                                     else 1
                                                     end
                    ) as total,


                    sum(
                      case ai.type when 'out_refund' then -1 
                                                     else 1
                                                     end
                    ) as total_invoices

                from
                  account_invoice as ai
                    left join
                  account_move_line as aml on ( ai.move_id = aml.move_id )
                    inner join
                  account_account as aa  on ( 
                      aml.account_id = aa.id and aa.type = 'receivable' 
                      )
                    inner join
                  (
                    select 
                      rp.id as partner_id,
                      min(extract(year from ai.date_invoice)) as year
                    from
                      res_partner as rp,
                      account_invoice as ai
                    where
                          rp.sales_report_exclude = False
                      and rp.id = ai.partner_id
                      and ai.type in('out_invoice')
                      and ai.state not in('cancel','draft')
                    group by 
                      rp.id
                  ) as pl
                  on (
                        pl.partner_id = ai.partner_id
                      )
                where
                  ai.type in('out_invoice','out_refund') 
                  and ai.state not in('cancel','draft')
                group by EXTRACT(YEAR FROM ai.date_invoice)
              ) as spy
              on ( spy.year = ncc.year )

            order by ncc.year

        ''')
        results = cr.fetchall()

        for yearly_data in results:
            new_yearly_record = {
                'year':           yearly_data[0],
                'new_total':      yearly_data[1],
                'total':          yearly_data[2],
                'new_partners':   yearly_data[3],
                'total_partners': yearly_data[4],
            }
            self.create(cr,uid,new_yearly_record,context)

zerp_yearly_sales()

