import time
import tools
import netsvc
from osv import fields,osv,orm
from tools import config

LIVE_SUBJECT_TEMPLATE = "[{case.id}] {case.name}"
LIVE_BODY_TEMPLATE = """
The responsible user is: {responsible.name}


If they are away someone else should resolve this issue. Whoever 
resolves it should close the case in ZERP. If it's an unusual 
problem consider adding a note to the case and replying-all to 
this email, indicating the problem and solution.

{body}
"""

DEBUG_SUBJECT_TEMPLATE = "[{case.id}] DEBUG {case.name}"
DEBUG_BODY_TEMPLATE = """
DEBUG/SANDBOX MODE: Normally these emails go to: {recipients}

The responsible user is: {responsible.name}

The database this sanity check resulted from was {cr.dbname}

{body}

If you are developing sanity checks and wish to acquire debug 
emails, include the following in your openerp server 
configuration:

sanity_check_force_emails_to=your@email.com

"""

class crm_helpdesk(osv.osv):
    _inherit = 'crm.helpdesk'

    ZABER_PARTNER_ID = 1
    ZABER_DEFAULT_CONTACT_ID = 9230 # Me, just for the time being
    SUBJECT_PREFIX = "SANITY_TEST: "

    def sanity_test_create_stub_case(
        self,
        cr,
        uid,
        context,
        subject,
        description,
        case_code):
        ''' Ensure that there's a stub record for the problem.
            This allows people to sign up for future notifications
            without having a record already there
        '''

        crm_section_db = self.pool.get('crm.case.section')
        paddr_db = self.pool.get('res.partner.address')

        massaged_subject = self.SUBJECT_PREFIX+subject
        sanity_case_code = 'sanity_case_' + case_code
        
        model_obj = self.pool.get('ir.model.data')
        entry_ids = model_obj.search(
            cr, 
            uid, 
            [('module', '=', 'zerp_sanity'), ('name','=', sanity_case_code)])
        if entry_ids:
            crm_match_id = model_obj.browse(
                cr, 
                uid, 
                entry_ids[0], 
                context).res_id
            if crm_match_id:
                # Check that the real record hasn't been deleted.
                id_exists = self.search(cr,uid,[('id','=',crm_match_id)],context)
                if id_exists: return crm_match_id
                    
        # Try searching by subject
        crm_match_ids = self.search(cr,uid,[
                          ('name','=',massaged_subject),
                          ('partner_id','=',self.ZABER_PARTNER_ID),
                      ])

        # Yes we do, we don't need to do any more
        if crm_match_ids:
            crm_match_id = crm_match_ids[0]
        else:
            # No existing crm case, we need to make it
            def_addr = paddr_db.browse(cr,uid,[self.ZABER_DEFAULT_CONTACT_ID])[0]
            #TODO: try state: open in original create()
            crm_match_id = self.create(cr,uid,{
                      'name': massaged_subject,
                      'email_from': def_addr.email,
                      'partner_id': self.ZABER_PARTNER_ID,
                      'state': 'done',
                      'description': description,
                      'partner_address_id': self.ZABER_DEFAULT_CONTACT_ID,
                  })

        if entry_ids:
            # Case must have been deleted, point entry to replacement case.
            model_obj.write(cr, uid, entry_ids, {'res_id': crm_match_id})
        else:
            model_obj.create(
                cr,
                uid,
                {
                    'name': sanity_case_code,
                    'module': 'zerp_sanity',
                    'model': 'crm.case',
                    'res_id': crm_match_id,
                    'noupdate': True
                })

        return crm_match_id

    def sanity_test_log (
        self,
        cr,
        uid,
        context,
        case_id,
        body,
        send_email=True,
        close_case=False):
        ''' Insert a record into the CRM Case history that 
            lists something about the latest panic
        '''


        case = self.browse(cr, uid, case_id)

        responsible_email = case.user_id.user_email
        emails = [responsible_email] + (case.email_cc or '').split(',')
        emails = filter(None, emails)
        if case.user_id.signature:
            body += '\n\n%s' % (case.user_id.signature)

        subject_template = LIVE_SUBJECT_TEMPLATE if cr.dbname == 'live' else DEBUG_SUBJECT_TEMPLATE;
        subject = subject_template.format(case=case)

        body_template = LIVE_BODY_TEMPLATE if cr.dbname == 'live' else DEBUG_BODY_TEMPLATE;
        body = body_template.format(
          cr=cr,
          body=body,
          responsible=case.user_id,
          recipients=", ".join(emails)
        )

        case_ids = [case_id]
        crm_rec = self.browse(cr,uid,case_id,context)
        if crm_rec.state == 'done':
            self.case_open(cr, uid, case_ids, context)

        # We don't want to spam people so we'll only send an email if the 
        # log itself changes. If things have not changed, we just silently
        # drop out, waiting for someone to fix things.
        if crm_rec.message_ids and crm_rec.message_ids[0].body_text == body:
           return

        # Now that we have a case header to work with, we're going to
        # append this new result to the history 
        mail_message = self.pool.get('mail.message')
        data = {
            'subject': 'Historize',
            'user_id': uid,
            'model' : self._name,
            'res_id': case_id,
            'date': time.strftime('%Y-%m-%d %H:%M:%S'),
            'body_text': self.format_body(body),
            'state' : 'received',
        }
        mail_message.create(cr, uid, data, context=context)

        if send_email:
            # Setting sanity_check_force_emails_to sets the sanity check
            # emails into debug mode and any database will send emails. However,
            # it will only send to the people configured on the system
            if config.get('sanity_check_force_emails_to',False):
                responsible_email = config.get('sanity_check_force_emails_to')
                emails = [responsible_email]

            # If we haven't put the database in debug mode, only send the email
            # if we're on the live server
            elif cr.dbname != 'live':
                emails = False

            if emails:
                self.pool.get('mail.message').schedule_with_attach(cr, uid,
                    responsible_email,
                    emails,
                    subject,
                    body,
                    model=self._name,
                    reply_to=case.section_id.reply_to,
                    res_id=case.id,
                    context=context
                )

        # If we want to simply close the case (log and not worry about it)
        if close_case:
            self.case_close(cr, uid, case_ids, context)

crm_helpdesk()

