from osv import fields
from osv import osv
from tools import config
from datetime import date
from datetime import datetime
from dateutil.relativedelta import relativedelta

class hr_contract(osv.osv):
    _inherit = 'hr.contract'

    def sanity_check(self, cr, uid, context=None):
        self.hr_work_permit(cr,uid,context)
        return 0

    def hr_work_permit(self,cr,uid,context):
        case_id = self.pool.get('crm.helpdesk').sanity_test_create_stub_case(cr,uid,context,
        "Found Work Permit expiring within 6 Months",
        """ This is a stub to track incidents where work permits will expire within 6 months. 
        """,
        'hr_work_permit')
        
        threshold_date = self.__months_ago(6)
        
        hr_emp_obj = self.pool.get('hr.employee')
        emp_ids = hr_emp_obj.search(cr, uid, [])
        
        if not emp_ids: return # okay, no employees
        
        emp_result = ""
        for emp in hr_emp_obj.browse(cr, uid, emp_ids):
            if not emp.contract_id.visa_expire: continue
            visa_expire = datetime.strptime(emp.contract_id.visa_expire, '%Y-%m-%d').date()
            if visa_expire <= threshold_date:
                emp_result += emp.name + " - " + str(visa_expire) + "\n"
                
        if emp_result == "": return # none expiring soon

        problem_writeup = "The following Employees have work permits expiring within 6 months:\n\n" + emp_result

        # notify appropriate users
        self.pool.get('crm.helpdesk').sanity_test_log(
                  cr, uid, context,
                  case_id,
                  problem_writeup
              )
              
    def __months_ago(self, months):
        " Calculate a timestamp from so many days ago. Returned as a string."
        
        return date.today() + relativedelta( months = months )

hr_contract()
