from osv import fields
from osv import osv
from tools import config


class purchase_order(osv.osv):
    _inherit = 'purchase.order'

    def sanity_check(self, cr, uid, context=None):
        self.sanity_check_fast(cr, uid, context)
        return 0

    def sanity_check_fast(self, cr, uid, context):
        ''' A set of fast sanity checks
        '''
        self.po_old_expected_date(cr,uid,context)
        self.po_invoice_exception(cr,uid,context)
        self.po_picking_exception(cr,uid,context)
        return
        
    def po_old_expected_date(self,cr,uid,context):
        case_id = self.pool.get('crm.helpdesk').sanity_test_create_stub_case(cr,uid,context,
        "Found Active POs with old expected dates",
        "This is a stub to track POs in the 'approved' or 'confirmed' state" +
        "with an expected date older than 2 weeks",
        'po_old_expected_date')
        
        cr.execute('''\
        SELECT
                    po.minimum_planned_date,
                    po.id,
                    po.name,
                    partner.name,
                    po.state
        FROM        purchase_order po
        LEFT JOIN   res_partner partner
              ON    po.partner_id = partner.id
        WHERE
                    now() - '14 days'::interval > po.minimum_planned_date and
                    state = 'approved'
        ORDER BY    po.minimum_planned_date
        ''')
        po_lines = cr.fetchall()
        if not po_lines:
            return # hurrah, no weirdness

        problem_writeup = "The following POs have been marked as having an old expected date:\n\n";
        for po in po_lines: #self.browse(cr,uid,po_ids):
            problem_writeup += "[%s] %s - %s - %s - %s\n" % po 
        problem_writeup += "\n\n";

        # now that we've finished documenting the problem let's notify the 
        # appropriate users
        self.pool.get('crm.helpdesk').sanity_test_log(
                  cr, uid, context,
                  case_id,
                  problem_writeup
              )

        return

    def po_invoice_exception(self,cr,uid,context):
        case_id = self.pool.get('crm.helpdesk').sanity_test_create_stub_case(cr,uid,context,
        "Found POs in a state of Invoice Exception",
        """ This is a stub to track incidents where POs are found with the status "Invoice Exception". 
        """,
        'po_invoice_exception')

        po_ids = self.search(cr,uid,[('state','=','except_invoice')])
        if not po_ids: return # okay, no problems

        problem_writeup = "The following POs have been marked as having an Invoice Exception:\n\n";
        for po in self.browse(cr,uid,po_ids):
            problem_writeup += "%i - %s - %s\n" % (po.id,po.name,po.partner_id.name)
        problem_writeup += "\n\n";

        # now that we've finished documenting the problem let's notify the 
        # appropriate users
        self.pool.get('crm.helpdesk').sanity_test_log(
                  cr, uid, context,
                  case_id,
                  problem_writeup
              )

        return


    def po_picking_exception(self,cr,uid,context):
        case_id = self.pool.get('crm.helpdesk').sanity_test_create_stub_case(cr,uid,context,
        "Found POs in a state of Shipping Exception",
        """ This is a stub to track incidents where POs are found with the status "Shipping Exception". 
        """,
        'po_picking_exception')

        po_ids = self.search(cr,uid,[('state','=','except_picking')])
        if not po_ids: return # okay, no problems

        problem_writeup = "The following POs have been marked as having an Shipping Exception:\n\n";
        for po in self.browse(cr,uid,po_ids):
            problem_writeup += "%i - %s - %s\n" % (po.id,po.name,po.partner_id.name)
        problem_writeup += "\n\n";

        # now that we've finished documenting the problem let's notify the 
        # appropriate users
        self.pool.get('crm.helpdesk').sanity_test_log(
                  cr, uid, context,
                  case_id,
                  problem_writeup
              )

        return

purchase_order()

from osv import fields
from osv import osv
from tools import config


class procurement_order(osv.osv):
    _inherit = 'procurement.order'

    def sanity_check(self, cr, uid, context=None):
        self.proc_order_state_lag(cr,uid,context)
        return 0
        
    def proc_order_state_lag(self,cr,uid,context):
        case_id = self.pool.get('crm.helpdesk').sanity_test_create_stub_case(cr,uid,context,
        "Found completed procurements marked as uncompleted",
        "This is a stub to track procurements that are not marked as done" +
        "but stock move/purchase order/production order are done",
        'proc_order_state_lag')
        
        cr.execute('''\
        SELECT
                    po.date_planned,
                    po.id,
                    po.state,
                    sm.id,
                    sm.state,
                    puo.id,
                    puo.state
        FROM        procurement_order po
        LEFT JOIN   stock_move sm
              ON    po.move_id = sm.id
        LEFT JOIN   purchase_order puo
              ON    po.purchase_id = puo.id
        WHERE
                    po.state != 'done' and 
                    po.state != 'cancel' and 
                    (sm.state = 'done' or
                    puo.state = 'done')
        ORDER BY    po.date_planned
        ''')
        po_lines = cr.fetchall()
        if not po_lines:
            return # hurrah, no weirdness

        problem_writeup = "The following procurements are not marked as completed but is completed in stock move or purchase order:\n\n";
        problem_writeup += "Stock Move Only:\n"
        for po in po_lines:
            if po[1] and not po[6]:
                problem_writeup += "%s - Proc|%s|%s| - Stock|%s|%s|\n" % (po[0], po[1], po[2], po[3], po[4])
        problem_writeup += "\n";
        
        problem_writeup += "Purchase Order Only:\n"
        for po in po_lines:
            if po[1] and not po[4]:
                problem_writeup += "%s - Proc|%s|%s| - PO|%s|%s|)\n" % (po[0], po[1], po[2], po[5], po[6])
        problem_writeup += "\n";
        
        problem_writeup += "Stock Move and Purchase Order:\n"
        for po in po_lines:
            if po[1] and po[6] and po[4]:
                problem_writeup += "%s - Proc|%s|%s| - Stock|%s|%s| - PO|%s|%s|\n" % (po[0], po[1], po[2], po[3], po[4], po[5], po[6])
        problem_writeup += "\n\n";
        
        # now that we've finished documenting the problem let's notify the 
        # appropriate users
        self.pool.get('crm.helpdesk').sanity_test_log(
                  cr, uid, context,
                  case_id,
                  problem_writeup
              )

        return


procurement_order()
