from osv import fields, osv
import datetime
import dateutil.parser
import logging
import urllib
import netsvc
from xml.etree.ElementTree import ElementTree,dump


class res_currency_rate ( osv.osv ):

    BOC_RATES_URL = "http://www.bankofcanada.ca/stats/assets/rates_rss/closing/en_usd.xml"
    CURRENCY_ID   = 4 # CAD

    _inherit = 'res.currency.rate'
    _columns = {}

    def boc_rates_fetch (self, cr, uid, data=False, context={}):
        '''Add support to automatically download the Bank of Canada's currency information
        '''

        logger = logging.getLogger('zerp_scheduled_tasks')
        

        # Try and fetch the RSS document
        try:
            rates_url = urllib.urlopen(self.BOC_RATES_URL)
        except:
            logger.warn("Could not contact remote host: "+self.BOC_RATES_URL)
            return

        # Parse the XML
        try:
            rates_tree = ElementTree()
            rates_tree.parse(rates_url)
        except:
            logger.warn("Something went wrong with the parsing of the CAD " +
                        "BoC document.")
            return

        # Uncomment this line if you'd like to see what Python
        # thinks the RSS XML tree looks like.
        # dump(rates_tree)

        # <ns4:observationPeriod frequency="daily">2011-06-14T16:15:00-04:00</ns4:observationPeriod>
        cad_usd_rate_date_element = rates_tree.find('.//{http://www.cbwiki.net/wiki/index.php/Specification_1.1}observationPeriod')
        if cad_usd_rate_date_element == None:
            logger.warn("Could not find date stamp in CAD BoC document.")
            return
        cad_usd_rate_date = dateutil.parser.parse(cad_usd_rate_date_element.text)

        # We add 1 to the day because for day X, we are using day X-1's closing rate
        cad_usd_rate_date += datetime.timedelta(1)

        # Find out what day it is. If the downloaded RSS feed is actually a bit older,
        # it means that the previous days had no currency update. So we just use the most 
        # current available.
        now = datetime.datetime.today()
        if cad_usd_rate_date.date() < now.date():
            cad_usd_rate_date = now

        date_ymd = cad_usd_rate_date.strftime("%Y-%m-%d")

        # <ns4:value decimals="4">1.0321</ns4:value>
        cad_usd_rate_element = rates_tree.find('.//{http://www.cbwiki.net/wiki/index.php/Specification_1.1}value')
        if cad_usd_rate_element == None:
            logger.warn("Could not find exchange rate value in CAD BoC " +
                        "document.")
            return
        # cad_usd_rate_element.text

        # Do we already have a value?
        matched_id = self.search(cr,uid,[
            ('currency_id','=',self.CURRENCY_ID),
            ('name','=',date_ymd)
        ])
        if len(matched_id) == 0:
            self.create(cr,uid,{
              'name': cad_usd_rate_date.date(),

              # note that we have to invert the rate to get the 
              # proper ratio (since we fix USD at 1)
              'rate': round(1/float(cad_usd_rate_element.text),4),
              'currency_id': self.CURRENCY_ID
            },context)

res_currency_rate()

