# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2012 Zaber Technologies Inc. (<http://zaber.com>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import netsvc
from osv import fields,osv,orm
import utils
from datetime import datetime

@utils.add_solr_support()
class crm_helpdesk(osv.osv):
    _inherit = "crm.helpdesk"

    def solr_record(self,cr,case,solr_rec):
        from crm import crm

        history_text = "\n".join([
            ch.body_text for ch in case.history_ids
                              if ch.body_text
        ])
        if case.description:
            history_text += "\n" + case.description

        country_name = "UNKNOWN"
        country = "UNKNOWN"
        if case.partner_address_id.country_id:
            country_name = case.partner_address_id.country_id.name
            country = case.partner_address_id.country_id.code
        elif case.partner_id.address:
            addresses = [ a for a in case.partner_id.address if a.country_id ]
            if addresses:
                addr_preferences = {
                    'invoice': 1,
                    'default': 2,
                    'delivery': 3,
                    'contact': 4,
                    'other': 5,
                }
                address = sorted(
                              addresses,
                              key=lambda a:addr_preferences.get(a.type,'10') 
                          )[0]
                country = address.country_id.code
                country_name = address.country_id.name

        if case.distributor_id:
            extra_partners = [case.distributor_id.name]
        else:
            extra_partners = []

        solr_rec.update({
          'email': case.email_from,
          'partner': case.partner_id.name,
          'extra_partners': ' '.join(extra_partners) or None,
          'contact': case.partner_address_id.name,
          'description': case.name,
          'responsible': case.user_id.name,
          'login': case.user_id.login,
          'history': history_text,
          'status': dict(crm.AVAILABLE_STATES)[case.state],
          'country': country,
          'country_name': country_name,
        })

        if case.date:
          solr_rec['date'] = datetime.strptime(case.date,"%Y-%m-%d %H:%M:%S"),

        return solr_rec


class mail_message(osv.osv):
    _inherit = 'mail.message'    
    _columns = {}

    def write(self, cr, uid, ids, *args, **kwargs):
        res = super(mail_message,self).write(cr,uid,ids,*args,**kwargs)

        case_ids = []
        for message in self.browse(cr,uid,ids):
            if id and message.model == 'crm.helpdesk':
                case_ids.append(message.res_id)
        if case_ids:
            self.pool.get('crm.helpdesk').solr_reindex_records(cr,uid,case_ids)
        return res

    def create(self, cr, uid, values, context=None):
        id = super(mail_message, self).create(cr, uid, values, context)

        if id and values.get('model') == 'crm.helpdesk':
            helpdesk_id = values['res_id']
            self.pool.get('crm.helpdesk').solr_reindex_records(cr,uid,[helpdesk_id])

        return id

    def unlink(self, cr, uid, ids, *args, **kwargs):
        case_ids = []
        for message in self.browse(cr,uid,ids):
            if id and message.model == 'crm.helpdesk':
                case_ids.append(message.res_id)
        res = super(mail_message,self).unlink(cr,uid,ids,*args,**kwargs)
        if case_ids:
            self.pool.get('crm.helpdesk').solr_reindex_records(cr,uid,list(case_ids))
        return res


