# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2012 Zaber Technologies Inc. (<http://zaber.com>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


import netsvc
from osv import fields,osv,orm
import utils
from datetime import datetime

@utils.add_solr_support()
class res_partner (osv.osv):
    _inherit = 'res.partner'

    def solr_reindex(self,cr,uid,context=None):
        self._solr_reindex(cr,uid,context)
        self.pool.get('res.partner.address').solr_reindex(cr,uid,context)

    def solr_record(self,cr,partner,solr_rec):

        description_list = []
        for key in (
              'comment',
              'order_comment',
              'shipping_account',
              'picking_warn_msg',
              'invoice_warn_msg',
              'purchase_warn_msg',
            ):
            description_list.append(partner[key])
        description = "\n".join(filter(None,description_list))

        # All the contacts associated with this partner
        contact_list = []
        for address in partner.address:
            contact = "\n".join(filter(None,[
                          address.name,
                          address.street,
                          address.street2,
                          address.email,
                          address.zip,
                          address.phone,
                          address.mobile,
                          address.fax,
                          address.where_found,
                          address.notes,
                        ]))
            contact_list.append(contact)

        contacts = "\n".join(filter(None,contact_list))

        name = partner.name
        if partner.alternate_names:
            name += "/" + partner.alternate_names

        solr_rec.update({
          'name': name,
          'partner': name,
          'description': description,
          'contacts': contacts,
          'country': partner.country.code,
          'country_name': partner.country.name,
        })

        if partner.supplier:
            solr_rec['supplier'] = 'yes'

        return solr_rec


@utils.add_solr_support()
class res_partner_address(osv.osv):
    _inherit = "res.partner.address"

    def solr_record(self,cr,address,solr_rec):

        description_list = []
        for key in (
              'street',
              'street2',
              'zip',
              'phone',
              'mobile',
              'fax',
              'where_found',
              'notes',
            ):
            description_list.append(address[key])
        description = "\n".join(filter(None,description_list))

        partner_name = address.partner_id.name
        if address.partner_id.alternate_names:
            partner_name += "/" + address.partner_id.alternate_names

        solr_rec.update({
          'name': address.name,
          'description': description,
          'type': address.type,
          'function': address.function,
          'country': address.country_id.code,
          'country_name': address.country_id.name,
          'email': address.email,
          'partner': partner_name,
        })

        return solr_rec


