from datetime import datetime, timedelta
from time import timezone as utcofs

from osv import fields, osv


class hr_timesheet_sheet(osv.osv):
    """
    Override hr_timesheet_sheet adding is_end_of_payperiod method and
    associated member. Used to trigger alert in timesheet view reminding
    users to submit their timesheets.
    """

    _inherit = "hr_timesheet_sheet.sheet"

    def _is_end_of_payperiod(self, cr, uid, ids, field_name, arg, context):
        """
        Method to calculate when last day of pay period is reached. Returns
        True on last day, otherwise False.

        TODO: Make sure date is correctly calculated taking timezone into
        account. date_to member is stored in UTC. date_current may be stored
        in localtime. According to documentation all dates are stored in UTC
        and server time is ignored.
            https://bugs.launchpad.net/openobject-server/+bug/970538

        :param cr:
        :param uid:
        :param ids:
        :param field_name:
        :param arg:
        :param context:
        """

        ret = {}
        data_obj = self.browse(cr, uid, ids)
        date_current = datetime.now() + timedelta(0, utcofs)
        for rec in data_obj:
            date_to = datetime.strptime(rec.date_to, "%Y-%m-%d")
            ret[rec.id] = (date_current >= date_to)
        return ret

    _columns = {
        "is_end_of_payperiod": fields.function(_is_end_of_payperiod,
                                               method=True, readonly=True,
                                               string="Is End of Pay Period",
                                               type='boolean')
    }
