from osv import osv

class sale_order(osv.osv):
    _inherit = 'sale.order'
    
    def order_confirm(self, cr, uid, ids, context=None):
        """ Check if an invoice needs to display a warning. If so, create
        the warning and ask for it to be displayed. If not, trigger the
        confirmation work flow. 
        """
        id, = ids
        sale = self.browse(cr, uid, id, context)

        # We only care about taxes when the customer is in our country
        company_country_id = (
            sale.company_id.partner_id.address[0].country_id.id)
        partner_country_id = sale.partner_shipping_id.country_id.id
        
        missing_taxes = False
        if company_country_id == partner_country_id:
            # So do all the line items have taxes associated with it?
            for line in sale.order_line:
                if not line.tax_id:
                    missing_taxes = True
                    break
        
        partner_name = sale.partner_id.name
        message = sale.partner_id.sale_warn_msg
        warning_level = sale.partner_id.sale_warn
        
        bad_products = []
        for line in sale.order_line:
            cannot_be_sold = line.product_id and not (
                line.product_id.active and
                line.product_id.state != 'obsolete')
            if cannot_be_sold:
                bad_products.append(line.product_id.default_code)
        if bad_products:
            warning_level = 'block'
            partner_name = None
            message = ('Product%s cannot be sold: %s' %
                       ('s' if len(bad_products) > 1 else '', 
                        ', '.join(bad_products)))
        
        return self.pool.get('confirm.warning').check_warning(
            cr, 
            uid, 
            'Confirm Sales Order', 
            partner_name, 
            message,
            self._name,
            id,
            'order_confirm',
            warning_level, 
            missing_taxes, 
            context)

sale_order()
